@@ -216,7 +216,7 @@ DEFINE_GUID(GUID_Buffer_Mono,                 0x186cc547,0xdb29,0x11d3,0x9b,0xd1
 typedef WORD    TRANSITION_TYPE, *LPTRANSITION_TYPE;
 /*
  * typedef __int64 REFERENCE_TIME,  *LPREFERENCE_TIME;
- * typedef long    MUSIC_TIME,      *LPMUSIC_TIME;
+ * typedef LONG    MUSIC_TIME,      *LPMUSIC_TIME;
  */
 
 
@@ -716,9 +716,9 @@ struct _DMUS_WAVE_PMSG {
 	DMUS_PMSG_PART    
 	REFERENCE_TIME rtStartOffset;
 	REFERENCE_TIME rtDuration;
-	long           lOffset;
-	long           lVolume;
-	long           lPitch;
+	LONG           lOffset;
+	LONG           lVolume;
+	LONG           lPitch;
 	BYTE           bFlags;
 };
 
@@ -1208,7 +1208,7 @@ DECLARE_INTERFACE_(IDirectMusicAudioPath,IUnknown)
     /*** IDirectMusicAudioPath methods ***/
     STDMETHOD(GetObjectInPath)(THIS_ DWORD dwPChannel, DWORD dwStage, DWORD dwBuffer, REFGUID guidObject, WORD dwIndex, REFGUID iidInterface, void **ppObject) PURE;
     STDMETHOD(Activate)(THIS_ BOOL fActivate) PURE;
-    STDMETHOD(SetVolume)(THIS_ long lVolume, DWORD dwDuration) PURE;
+    STDMETHOD(SetVolume)(THIS_ LONG lVolume, DWORD dwDuration) PURE;
     STDMETHOD(ConvertPChannel)(THIS_ DWORD dwPChannelIn, DWORD *pdwPChannelOut) PURE;
 };
 #undef INTERFACE
