@@ -294,6 +294,19 @@ BOOL 		WINAPI 	PlaySoundW(LPCWSTR pszSound, HMODULE hmod, DWORD fdwSound);
 #define SND_PURGE		0x00000040L /* purge all sounds */
 #define SND_APPLICATION     	0x00000080L /* look for application specific association */
 
+#define SND_ALIAS_START         0
+
+#define sndAlias(ch0, ch1)              (SND_ALIAS_START + ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8)))
+
+#define SND_ALIAS_SYSTEMASTERISK        sndAlias('S', '*')
+#define SND_ALIAS_SYSTEMQUESTION        sndAlias('S', '?')
+#define SND_ALIAS_SYSTEMHAND            sndAlias('S', 'H')
+#define SND_ALIAS_SYSTEMEXIT            sndAlias('S', 'E')
+#define SND_ALIAS_SYSTEMSTART           sndAlias('S', 'S')
+#define SND_ALIAS_SYSTEMWELCOME         sndAlias('S', 'W')
+#define SND_ALIAS_SYSTEMEXCLAMATION     sndAlias('S', '!')
+#define SND_ALIAS_SYSTEMDEFAULT         sndAlias('S', 'D')
+
 /* waveform audio error return values */
 #define WAVERR_BADFORMAT      (WAVERR_BASE + 0)    /* unsupported wave format */
 #define WAVERR_STILLPLAYING   (WAVERR_BASE + 1)    /* still something playing */
@@ -322,11 +335,11 @@ typedef struct wavehdr_tag {
     LPSTR       lpData;
     DWORD       dwBufferLength;
     DWORD       dwBytesRecorded;
-    DWORD       dwUser;
+    DWORD_PTR   dwUser;
     DWORD       dwFlags;
     DWORD       dwLoops;
     struct wavehdr_tag *lpNext;
-    DWORD       reserved;
+    DWORD_PTR   reserved;
 } WAVEHDR, *PWAVEHDR, *NPWAVEHDR, *LPWAVEHDR;
 
 #define WHDR_DONE       0x00000001
@@ -713,7 +726,7 @@ typedef struct midihdr_tag {
     DWORD_PTR	dwUser;
     DWORD	dwFlags;
     struct midihdr_tag *lpNext;
-    DWORD	reserved;
+    DWORD_PTR	reserved;
     DWORD	dwOffset;
     DWORD_PTR	dwReserved[8];
 } MIDIHDR, *LPMIDIHDR;
@@ -1234,7 +1247,7 @@ typedef struct tagMIXERLINEA {
     DWORD	dwSource;
     DWORD	dwLineID;
     DWORD	fdwLine;
-    DWORD	dwUser;
+    DWORD_PTR	dwUser;
     DWORD	dwComponentType;
     DWORD	cChannels;
     DWORD	cConnections;
@@ -1257,7 +1270,7 @@ typedef struct tagMIXERLINEW {
     DWORD	dwSource;
     DWORD	dwLineID;
     DWORD	fdwLine;
-    DWORD	dwUser;
+    DWORD_PTR	dwUser;
     DWORD	dwComponentType;
     DWORD	cChannels;
     DWORD	cConnections;
@@ -1986,7 +1999,7 @@ typedef struct tagMCI_SEEK_PARMS {
 
 typedef struct tagMCI_STATUS_PARMS {
        DWORD_PTR dwCallback;
-	DWORD   dwReturn;
+       DWORD_PTR dwReturn;
 	DWORD   dwItem;
 	DWORD   dwTrack;
 } MCI_STATUS_PARMS, *LPMCI_STATUS_PARMS;
