@@ -166,10 +166,10 @@ typedef struct tagPICTDESC {
 	} DUMMYUNIONNAME;
 } PICTDESC, *LPPICTDESC;
 
-typedef long OLE_XPOS_PIXELS;
-typedef long OLE_YPOS_PIXELS;
-typedef long OLE_XSIZE_PIXELS;
-typedef long OLE_YSIZE_PIXELS;
+typedef LONG OLE_XPOS_PIXELS;
+typedef LONG OLE_YPOS_PIXELS;
+typedef LONG OLE_XSIZE_PIXELS;
+typedef LONG OLE_YSIZE_PIXELS;
 typedef float OLE_XPOS_CONTAINER;
 typedef float OLE_YPOS_CONTAINER;
 typedef float OLE_XSIZE_CONTAINER;
