@@ -525,7 +525,7 @@ DECL_WINELIB_TYPE_AW(TEXTRANGE)
 
 typedef struct _editstream
 {
-    DWORD		dwCookie;
+    DWORD_PTR		dwCookie;
     DWORD		dwError;
     EDITSTREAMCALLBACK	pfnCallback;
 } EDITSTREAM;
@@ -631,7 +631,7 @@ typedef enum tagKHYPH
 typedef struct hyphresult
 {
     KHYPH      khyph;
-    long       ichHyph;
+    LONG       ichHyph;
     WCHAR      chHyph;
 } HYPHRESULT;
 
@@ -639,7 +639,7 @@ typedef struct tagHyphenateInfo
 {
     SHORT      cbSize;
     SHORT      dxHyphenateZone;
-    void       (WINAPI* pfnHyphenate)(WCHAR*, LANGID, long, HYPHRESULT*);
+    void       (WINAPI* pfnHyphenate)(WCHAR*, LANGID, LONG, HYPHRESULT*);
 } HYPHENATEINFO;
 
 typedef struct _msgfilter {
@@ -721,7 +721,7 @@ typedef struct _imecomptext {
     DWORD      flags;
 } IMECOMPTEXT;
 
-void WINAPI HyphenateProc(WCHAR*, LANGID, long, HYPHRESULT*);
+void WINAPI HyphenateProc(WCHAR*, LANGID, LONG, HYPHRESULT*);
 
 #define SF_TEXT		      0x00000001
 #define SF_RTF		      0x00000002
