@@ -31,8 +31,10 @@
 
 #include <commctrl.h>
 
-#ifdef __cplusplus
-extern "C" {
+#ifdef _WIN64
+#include <pshpack8.h>
+#else
+#include <pshpack1.h>
 #endif
 
 /* setupapi doesn't use the normal convention, it adds an underscore before A/W */
@@ -1223,6 +1225,7 @@ DECL_WINELIB_SETUPAPI_TYPE_AW(PSP_INF_SIGNER_INFO)
 #define ERROR_SET_SYSTEM_RESTORE_POINT    (APPLICATION_ERROR_MASK|ERROR_SEVERITY_ERROR|0x236)
 #define ERROR_INCORRECTLY_COPIED_INF      (APPLICATION_ERROR_MASK|ERROR_SEVERITY_ERROR|0x237)
 #define ERROR_SCE_DISABLED                (APPLICATION_ERROR_MASK|ERROR_SEVERITY_ERROR|0x238)
+#define ERROR_WRONG_INF_TYPE              (APPLICATION_ERROR_MASK|ERROR_SEVERITY_ERROR|0x24A)
 #define ERROR_NO_DEFAULT_INTERFACE_DEVICE ERROR_NO_DEFAULT_DEVICE_INTERFACE
 #define ERROR_INTERFACE_DEVICE_ACTIVE     ERROR_DEVICE_INTERFACE_ACTIVE
 #define ERROR_INTERFACE_DEVICE_REMOVED    ERROR_DEVICE_INTERFACE_REMOVED
@@ -1301,11 +1304,11 @@ DECL_WINELIB_SETUPAPI_TYPE_AW(PSP_INF_SIGNER_INFO)
 #define SPDRP_INSTALL_STATE               0x00000022
 #define SPDRP_MAXIMUM_PROPERTY            0x00000023
 
-#define DPROMPT_SUCCESS       0
-#define DPROMPT_CANCEL        1
-#define DPROMPT_SKIPFILE      2
-#define DPROMPT_BUFFERTOSMALL 3
-#define DPROMPT_OUTOFMEMORY   4
+#define DPROMPT_SUCCESS        0
+#define DPROMPT_CANCEL         1
+#define DPROMPT_SKIPFILE       2
+#define DPROMPT_BUFFERTOOSMALL 3
+#define DPROMPT_OUTOFMEMORY    4
 
 #define SETDIRID_NOT_FULL_PATH 0x00000001
 
@@ -1318,6 +1321,10 @@ DECL_WINELIB_SETUPAPI_TYPE_AW(PSP_INF_SIGNER_INFO)
 #define IDF_NOFOREGROUND 0x00000400
 #define IDF_WARNIFSKIP   0x00000800
 
+#define IDF_NOREMOVABLEMEDIAPROMPT 0x00001000
+#define IDF_USEDISKNAMEASPROMPT    0x00002000
+#define IDF_OEMDISK                0x80000000
+
 #define INFINFO_INF_SPEC_IS_HINF        1
 #define INFINFO_INF_NAME_IS_ABSOLUTE    2
 #define INFINFO_DEFAULT_SEARCH          3
@@ -1384,6 +1391,9 @@ typedef enum {
 
 /* SetupDiGetWizardPage PageType values */
 #define SPWPT_SELECTDEVICE 0x00000001
+/* SetupUninstallOEMInf Flags values */
+#define SUOI_FORCEDELETE 0x00000001
+
 /* SetupDiGetWizardPage Flags values */
 #define SPWP_USE_DEVINFO_DATA 0x00000001
 
@@ -1641,8 +1651,8 @@ BOOL     WINAPI SetupDiUnremoveDevice(HDEVINFO, PSP_DEVINFO_DATA);
 HDSKSPC  WINAPI SetupDuplicateDiskSpaceListA(HDSKSPC, PVOID, DWORD, UINT);
 HDSKSPC  WINAPI SetupDuplicateDiskSpaceListW(HDSKSPC, PVOID, DWORD, UINT);
 #define         SetupDuplicateDiskSpaceList WINELIB_NAME_AW(SetupDuplicateDiskSpaceList)
-BOOL     WINAPI SetupEnumInfSectionsA(HINF, UINT, PSTR, SIZE, UINT *);
-BOOL     WINAPI SetupEnumInfSectionsW(HINF, UINT, PWSTR, SIZE, UINT *);
+BOOL     WINAPI SetupEnumInfSectionsA(HINF, UINT, PSTR, DWORD, DWORD *);
+BOOL     WINAPI SetupEnumInfSectionsW(HINF, UINT, PWSTR, DWORD, DWORD *);
 #define         SetupEnumInfSections WINELIB_NAME_AW(SetupEnumInfSections)
 BOOL     WINAPI SetupFindFirstLineA( HINF hinf, PCSTR section, PCSTR key, INFCONTEXT *context );
 BOOL     WINAPI SetupFindFirstLineW( HINF hinf, PCWSTR section, PCWSTR key, INFCONTEXT *context );
@@ -1856,8 +1866,6 @@ BOOL     WINAPI UnmapAndCloseFile(HANDLE, HANDLE, PVOID);
 
 #undef DECL_WINELIB_SETUPAPI_TYPE_AW
 
-#ifdef __cplusplus
-}
-#endif
+#include <poppack.h>
 
 #endif /* _INC_SETUPAPI */
