@@ -50,6 +50,8 @@
 #include "wine/library.h"
 #include "wine/list.h"
 
+#ifdef HAVE_MMAP
+
 struct reserved_area
 {
     struct list entry;
@@ -60,8 +62,6 @@ struct reserved_area
 static struct list reserved_areas = LIST_INIT(reserved_areas);
 static const unsigned int granularity_mask = 0xffff;  /* reserved areas have 64k granularity */
 
-#ifdef HAVE_MMAP
-
 #ifndef MAP_NORESERVE
 #define MAP_NORESERVE 0
 #endif
@@ -252,7 +252,7 @@ static inline int mmap_reserve( void *addr, size_t size )
  *
  * Reserve as much memory as possible in the given area.
  */
-#if defined(__i386__) && !defined(__FreeBSD__) && !defined(__FreeBSD_kernel__)  /* commented out until FreeBSD gets fixed */
+#ifdef __i386__
 static void reserve_area( void *addr, void *end )
 {
     size_t size = (char *)end - (char *)addr;
@@ -352,7 +352,7 @@ void mmap_init(void)
 {
     struct reserved_area *area;
     struct list *ptr;
-#if defined(__i386__) && !defined(__FreeBSD__) && !defined(__FreeBSD_kernel__)  /* commented out until FreeBSD gets fixed */
+#ifdef __i386__
     char stack;
     char * const stack_ptr = &stack;
     char *user_space_limit = (char *)0x7ffe0000;
@@ -378,8 +378,8 @@ void mmap_init(void)
         char *base = stack_ptr - ((unsigned int)stack_ptr & granularity_mask) - (granularity_mask + 1);
         if (base > user_space_limit) reserve_area( user_space_limit, base );
         base = stack_ptr - ((unsigned int)stack_ptr & granularity_mask) + (granularity_mask + 1);
-#ifdef linux
-        /* Linux heuristic: assume the stack is near the end of the address */
+#if defined(linux) || defined(__FreeBSD__)
+        /* Heuristic: assume the stack is near the end of the address */
         /* space, this avoids a lot of futile allocation attempts */
         end = (char *)(((unsigned long)base + 0x0fffffff) & 0xf0000000);
 #endif
@@ -399,23 +399,6 @@ void mmap_init(void)
     reserve_dos_area();
 }
 
-#else /* HAVE_MMAP */
-
-void *wine_anon_mmap( void *start, size_t size, int prot, int flags )
-{
-    return (void *)-1;
-}
-
-static inline int munmap( void *ptr, size_t size )
-{
-    return 0;
-}
-
-void mmap_init(void)
-{
-}
-
-#endif
 
 /***********************************************************************
  *           wine_mmap_add_reserved_area
@@ -611,3 +594,11 @@ int wine_mmap_enum_reserved_areas( int (*enum_func)(void *base, size_t size, voi
     }
     return ret;
 }
+
+#else /* HAVE_MMAP */
+
+void mmap_init(void)
+{
+}
+
+#endif
