@@ -27,7 +27,8 @@
 
 #include <iprt/assert.h>
 #if !defined(RT_OS_WINDOWS) && !defined(RT_OS_LINUX)
-#include <iprt/memobj.h>
+# include <iprt/memobj.h>
+# include <iprt/mem.h>
 #endif
 
 
@@ -92,8 +93,9 @@ int vbglLockLinear (void **ppvCtx, void *pv, uint32_t u32Size, bool fWriteAccess
 
 #else
     /* Default to IPRT - this ASSUMES that it is USER addresses we're locking. */
-    RTR0MEMOBJ MemObj;
-    rc = RTR0MemObjLockUser(&MemObj, (RTR3PTR)pv, u32Size, NIL_RTR0PROCESS);
+    RTR0MEMOBJ MemObj = NIL_RTR0MEMOBJ;
+    uint32_t fAccess = RTMEM_PROT_READ | (fWriteAccess ? RTMEM_PROT_WRITE : 0);
+    rc = RTR0MemObjLockUser(&MemObj, (RTR3PTR)pv, u32Size, fAccess, NIL_RTR0PROCESS);
     if (RT_SUCCESS(rc))
         *ppvCtx = MemObj;
     else
@@ -141,37 +143,37 @@ void vbglUnlockLinear (void *pvCtx, void *pv, uint32_t u32Size)
 #endif
 
 #ifdef RT_OS_LINUX
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 extern DECLVBGL(void *) vboxadd_cmc_open (void);
 extern DECLVBGL(void) vboxadd_cmc_close (void *);
 extern DECLVBGL(int) vboxadd_cmc_call (void *opaque, uint32_t func, void *data);
-__END_DECLS
+RT_C_DECLS_END
 #endif /* RT_OS_LINUX */
 
 #ifdef RT_OS_OS2
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 /*
  * On OS/2 we'll do the connecting in the assembly code of the
  * client driver, exporting a g_VBoxGuestIDC symbol containing
  * the connection information obtained from the 16-bit IDC.
  */
 extern VBOXGUESTOS2IDCCONNECT g_VBoxGuestIDC;
-__END_DECLS
+RT_C_DECLS_END
 #endif
 
 #ifdef RT_OS_SOLARIS
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 extern DECLVBGL(void *) VBoxGuestSolarisServiceOpen (uint32_t *pu32Version);
 extern DECLVBGL(void) VBoxGuestSolarisServiceClose (void *pvOpaque);
 extern DECLVBGL(int) VBoxGuestSolarisServiceCall (void *pvOpaque, unsigned int iCmd, void *pvData, size_t cbSize, size_t *pcbReturn);
-__END_DECLS
+RT_C_DECLS_END
 
 #elif defined (RT_OS_FREEBSD)
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 extern DECLVBGL(void *) VBoxGuestFreeBSDServiceOpen (uint32_t *pu32Version);
 extern DECLVBGL(void) VBoxGuestFreeBSDServiceClose (void *pvOpaque);
 extern DECLVBGL(int) VBoxGuestFreeBSDServiceCall (void *pvOpaque, unsigned int iCmd, void *pvData, size_t cbSize, size_t *pcbReturn);
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
