@@ -798,8 +798,8 @@ static int vboxadd_ioctl(struct inode *inode, struct file *filp,
     {
         VMMDevRequestHeader reqHeader;
         VMMDevRequestHeader *reqFull = NULL;
-        size_t cbRequestSize;
-        size_t cbVanillaRequestSize;
+        size_t cbRequestSize = 0;
+        size_t cbVanillaRequestSize = 0;
 
         IOCTL_VMM_ENTRY(arg);
         if (copy_from_user(&reqHeader, (void*)arg, sizeof(reqHeader)))
@@ -831,6 +831,21 @@ static int vboxadd_ioctl(struct inode *inode, struct file *filp,
             }
         }
         if (0 == rc)
+        {
+            rc = VbglGRVerify(&reqHeader, cbRequestSize);
+            if (RT_FAILURE(rc))
+            {
+                Log(("VBOXGUEST_IOCTL_VMMREQUEST: invalid request header: size %d min: %d type: %d rc: %d\n",
+                     cbRequestSize,
+                     cbVanillaRequestSize,
+                     reqHeader.requestType,
+                     rc));
+                rc = -EINVAL;
+            }
+            else
+                rc = 0;
+        }
+        if (0 == rc)
         {
             /* request storage for the full request */
             rc = VbglGRAlloc(&reqFull, cbRequestSize, reqHeader.requestType);
@@ -961,8 +976,6 @@ static int vboxuser_ioctl(struct inode *inode, struct file *filp,
     int rc = 0;
 
     /* Deal with variable size ioctls first. */
-#ifdef DEBUG  /* Only allow random user applications to spam the log in
-               * debug additions builds */
     if (   VBOXGUEST_IOCTL_STRIP_SIZE(VBOXGUEST_IOCTL_LOG(0))
         == VBOXGUEST_IOCTL_STRIP_SIZE(cmd))
     {
@@ -983,24 +996,21 @@ static int vboxuser_ioctl(struct inode *inode, struct file *filp,
             rc = -EFAULT;
         }
         if (0 == rc)
-        {
+            /* This only produces output in debug builds */
             Log(("%.*s", _IOC_SIZE(cmd), pszMessage));
-        }
         if (NULL != pszMessage)
         {
             kfree(pszMessage);
         }
         IOCTL_LOG_EXIT(arg);
     }
-    else
-#endif
-    if (   VBOXGUEST_IOCTL_STRIP_SIZE(VBOXGUEST_IOCTL_VMMREQUEST(0))
+    else if (   VBOXGUEST_IOCTL_STRIP_SIZE(VBOXGUEST_IOCTL_VMMREQUEST(0))
              == VBOXGUEST_IOCTL_STRIP_SIZE(cmd))
     {
         VMMDevRequestHeader reqHeader;
         VMMDevRequestHeader *reqFull = NULL;
-        size_t cbRequestSize;
-        size_t cbVanillaRequestSize;
+        size_t cbRequestSize = 0;
+        size_t cbVanillaRequestSize = 0;
 
         IOCTL_VMM_ENTRY(arg);
         if (copy_from_user(&reqHeader, (void*)arg, sizeof(reqHeader)))
@@ -1032,6 +1042,21 @@ static int vboxuser_ioctl(struct inode *inode, struct file *filp,
             }
         }
         if (0 == rc)
+        {
+            rc = VbglGRVerify(&reqHeader, cbRequestSize);
+            if (RT_FAILURE(rc))
+            {
+                Log(("VBOXGUEST_IOCTL_VMMREQUEST: invalid request header: size %d min: %d type: %d rc: %d\n",
+                     cbRequestSize,
+                     cbVanillaRequestSize,
+                     reqHeader.requestType,
+                     rc));
+                rc = -EINVAL;
+            }
+            else
+                rc = 0;
+        }
+        if (0 == rc)
         {
             /* request storage for the full request */
             rc = VbglGRAlloc(&reqFull, cbRequestSize, reqHeader.requestType);
