@@ -1,4 +1,4 @@
-/** $Id: VBoxSFInit.cpp 3655 2007-07-16 18:47:26Z vboxsync $ */
+/** $Id: VBoxSFInit.cpp $ */
 /** @file
  * VBoxSF - OS/2 Shared Folders, Initialization.
  */
@@ -45,14 +45,14 @@
 /*******************************************************************************
 *   Global Variables                                                           *
 *******************************************************************************/
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 /* from VBoxSFA.asm */
 extern RTFAR16 g_fpfnDevHlp;
 extern VBOXGUESTOS2IDCCONNECT g_VBoxGuestIDC;
 extern uint32_t g_u32Info;
 /* from sys0.asm and the linker/end.lib. */
 extern char _text, _etext, _data, _end;
-__END_DECLS
+RT_C_DECLS_END
 
 
 
