@@ -214,7 +214,7 @@ typedef PCIGLOBALS *PPCIGLOBALS;
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 PDMBOTHCBDECL(void) pciSetIrq(PPDMDEVINS pDevIns, PPCIDEVICE pPciDev, int iIrq, int iLevel);
 PDMBOTHCBDECL(void) pcibridgeSetIrq(PPDMDEVINS pDevIns, PPCIDEVICE pPciDev, int iIrq, int iLevel);
@@ -227,7 +227,7 @@ PDMBOTHCBDECL(int)  pciIOPortDataRead(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT
 DECLINLINE(PPCIDEVICE) pciFindBridge(PPCIBUS pBus, uint8_t iBus);
 #endif
 
-__END_DECLS
+RT_C_DECLS_END
 
 #define DEBUG_PCI
 
@@ -932,6 +932,14 @@ static void pci_bios_init_device(PPCIGLOBALS pGlobals, uint8_t uBus, uint8_t uDe
                     goto default_map;
                 /* VGA: map frame buffer to default Bochs VBE address */
                 pci_set_io_region_addr(pGlobals, uBus, uDevFn, 0, 0xE0000000);
+#ifdef VBOX_WITH_EFI
+                /* The following is necessary for the VGA check in
+                   PciVgaMiniPortDriverBindingSupported to succeed. */
+                /** @todo Seems we're missing some I/O registers or something on the VGA device... Compare real/virt hw with lspci. */
+                pci_config_writew(pGlobals, uBus, uDevFn, PCI_COMMAND,
+                                  pci_config_readw(pGlobals, uBus, uDevFn, PCI_COMMAND)
+                                  | 1 /* Enable I/O space access. */);
+#endif
                 break;
             case 0x0800:
                 /* PIC */
@@ -1256,7 +1264,7 @@ static DECLCALLBACK(int) pciGenericLoadExec(PPDMDEVINS pDevIns, PPCIDEVICE pPciD
  * @param   pPciDev     Pointer to PCI device.
  * @param   pSSMHandle  The handle to save the state to.
  */
-static DECLCALLBACK(int) pciSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
+static DECLCALLBACK(int) pciR3SaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
 {
     uint32_t    i;
     PPCIGLOBALS pThis = PDMINS_2_DATA(pDevIns, PPCIGLOBALS);
@@ -1282,7 +1290,7 @@ static DECLCALLBACK(int) pciSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
     SSMR3PutU32(pSSMHandle, ~0);        /* separator */
 
     /*
-     * Iterate all the devices.
+     * Iterate thru all the devices.
      */
     for (i = 0; i < RT_ELEMENTS(pBus->devices); i++)
     {
@@ -1327,7 +1335,6 @@ static void pciR3CommonRestoreConfig(PPCIDEVICE pDev, uint8_t const *pbSrcConfig
         /* off,cb,fW,fB, pszName */
         { 0x00, 2, 0, 3, "VENDOR_ID" },
         { 0x02, 2, 0, 3, "DEVICE_ID" },
-        { 0x04, 2, 1, 3, "COMMAND" },
         { 0x06, 2, 1, 3, "STATUS" },
         { 0x08, 1, 0, 3, "REVISION_ID" },
         { 0x09, 1, 0, 3, "CLASS_PROG" },
@@ -1370,6 +1377,10 @@ static void pciR3CommonRestoreConfig(PPCIDEVICE pDev, uint8_t const *pbSrcConfig
         { 0x3e, 1, 0, 1, "MIN_GNT" },           // fWritable = !?
         { 0x3e, 1, 1, 2, "BRIDGE_CONTROL" },    // fWritable = !? cb=!?
         { 0x3f, 1, 1, 3, "MAX_LAT" },           // fWritable = !? fBridge=!?
+        /* The COMMAND register must come last as it requires the *ADDRESS*
+           registers to be restored before we pretent to change it from 0 to
+           whatever value the guest assigned it. */
+        { 0x04, 2, 1, 3, "COMMAND" },
     };
 
 #ifdef RT_STRICT
@@ -1430,14 +1441,20 @@ static void pciR3CommonRestoreConfig(PPCIDEVICE pDev, uint8_t const *pbSrcConfig
                     continue;
             }
 
-            if (u32Src != u32Dst)
+            if (    u32Src != u32Dst
+                ||  off == VBOX_PCI_COMMAND)
             {
-                if (!s_aFields[i].fWritable)
-                    LogRel(("PCI: %8s/%u: %2u-bit field %s: %x -> %x - !READ ONLY!\n",
-                            pDev->name, pDev->pDevIns->iInstance, cb*8, s_aFields[i].pszName, u32Dst, u32Src));
-                else
-                    LogRel(("PCI: %8s/%u: %2u-bit field %s: %x -> %x\n",
-                            pDev->name, pDev->pDevIns->iInstance, cb*8, s_aFields[i].pszName, u32Dst, u32Src));
+                if (u32Src != u32Dst)
+                {
+                    if (!s_aFields[i].fWritable)
+                        LogRel(("PCI: %8s/%u: %2u-bit field %s: %x -> %x - !READ ONLY!\n",
+                                pDev->name, pDev->pDevIns->iInstance, cb*8, s_aFields[i].pszName, u32Dst, u32Src));
+                    else
+                        LogRel(("PCI: %8s/%u: %2u-bit field %s: %x -> %x\n",
+                                pDev->name, pDev->pDevIns->iInstance, cb*8, s_aFields[i].pszName, u32Dst, u32Src));
+                }
+                if (off == VBOX_PCI_COMMAND)
+                    PCIDevSetCommand(pDev, 0); /* For remapping, see pciR3CommonLoadExec. */
                 pDev->Int.s.pfnConfigWrite(pDev, off, u32Src, cb);
             }
         }
@@ -1467,7 +1484,7 @@ static void pciR3CommonRestoreConfig(PPCIDEVICE pDev, uint8_t const *pbSrcConfig
  * @param   pSSMHandle  The handle to the saved state.
  * @param   u32Version  The data unit version number.
  */
-static DECLCALLBACK(int) pciLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle, uint32_t u32Version)
+static DECLCALLBACK(int) pciR3LoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle, uint32_t u32Version)
 {
     PPCIGLOBALS pThis = PDMINS_2_DATA(pDevIns, PPCIGLOBALS);
     PPCIBUS     pBus  = &pThis->PciBus;
@@ -1510,6 +1527,26 @@ static DECLCALLBACK(int) pciLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle,
     if (u32 != (uint32_t)~0)
         AssertMsgFailedReturn(("u32=%#x\n", u32), rc);
 
+    /*
+     * Iterate thru all the devices and write 0 to the COMMAND register so
+     * that all the memory is unmapped before we start restoring the saved
+     * mapping locations.
+     *
+     * The register value is restored afterwards so we can do proper
+     * LogRels in pciR3CommonRestoreConfig.
+     */
+    for (uint32_t i = 0; i < RT_ELEMENTS(pBus->devices); i++)
+    {
+        PPCIDEVICE pDev = pBus->devices[i];
+        if (pDev)
+        {
+            uint16_t u16 = PCIDevGetCommand(pDev);
+            pDev->Int.s.pfnConfigWrite(pDev, VBOX_PCI_COMMAND, 0, 2);
+            PCIDevSetCommand(pDev, u16);
+            Assert(PCIDevGetCommand(pDev) == u16);
+        }
+    }
+
     /*
      * Iterate all the devices.
      */
@@ -1841,7 +1878,7 @@ static DECLCALLBACK(int) pciFakePCIBIOS(PPDMDEVINS pDevIns)
     /*
      * Set the start addresses.
      */
-    pGlobals->pci_bios_io_addr  = 0xc000;
+    pGlobals->pci_bios_io_addr  = 0xd000;
     pGlobals->pci_bios_mem_addr = UINT32_C(0xf0000000);
     pGlobals->uBus = 0;
 
@@ -2051,7 +2088,7 @@ static DECLCALLBACK(int)   pciConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGM
     }
 
     rc = SSMR3RegisterDevice(PDMDevHlpGetVM(pDevIns), pDevIns, "pci", iInstance, VBOX_PCI_SAVED_STATE_VERSION, sizeof(*pBus) + 16*128, "pgm",
-                             NULL, pciSaveExec, NULL, NULL, pciLoadExec, NULL);
+                             NULL, pciR3SaveExec, NULL, NULL, pciR3LoadExec, NULL);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -2219,8 +2256,9 @@ static uint32_t pcibridgeConfigRead(PPDMDEVINSR3 pDevIns, uint8_t iBus, uint8_t
  * @param   pPciDev     Pointer to PCI device.
  * @param   pSSMHandle  The handle to save the state to.
  */
-static DECLCALLBACK(int) pcibridgeSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
+static DECLCALLBACK(int) pcibridgeR3SaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
 {
+/** @todo make common with pciR3SaveExec! */
     uint32_t    i;
     PPCIBUS pThis = PDMINS_2_DATA(pDevIns, PPCIBUS);
 
@@ -2244,6 +2282,7 @@ static DECLCALLBACK(int) pcibridgeSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHa
     return SSMR3PutU32(pSSMHandle, ~0); /* terminator */
 }
 
+
 /**
  * Loads a saved PCI bridge device state.
  *
@@ -2252,14 +2291,14 @@ static DECLCALLBACK(int) pcibridgeSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHa
  * @param   pSSMHandle  The handle to the saved state.
  * @param   u32Version  The data unit version number.
  */
-static DECLCALLBACK(int) pcibridgeLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle, uint32_t u32Version)
+static DECLCALLBACK(int) pcibridgeR3LoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle, uint32_t u32Version)
 {
     PPCIBUS     pBus  = PDMINS_2_DATA(pDevIns, PPCIBUS);
     uint32_t    u32;
     uint32_t    i;
     int         rc;
 
-/** @todo r=bird: this is a copy of pciLoadExec. combine the two!  */
+/** @todo r=bird: this is a copy of pciR3LoadExec. combine the two!  */
 
     /*
      * Check the version.
@@ -2518,7 +2557,7 @@ static DECLCALLBACK(int)   pcibridgeConstruct(PPDMDEVINS pDevIns, int iInstance,
      * to make changes easier.
      */
     rc = SSMR3RegisterDevice(PDMDevHlpGetVM(pDevIns), pDevIns, "pcibridge", iInstance, VBOX_PCI_SAVED_STATE_VERSION, sizeof(*pBus) + 16*128, "pgm",
-                             NULL, pcibridgeSaveExec, NULL, NULL, pcibridgeLoadExec, NULL);
+                             NULL, pcibridgeR3SaveExec, NULL, NULL, pcibridgeR3LoadExec, NULL);
     if (RT_FAILURE(rc))
         return rc;
 
