@@ -182,7 +182,7 @@ static DECLCALLBACK(int) drvHostBaseRead(PPDMIBLOCK pInterface, uint64_t off, vo
             off    += cbRead32;
             cbRead -= cbRead32;
             pvBuf   = (uint8_t *)pvBuf + cbRead32;
-        } while ((cbRead > 0) && RT_SUCCESS(rc)); 
+        } while ((cbRead > 0) && RT_SUCCESS(rc));
 
 #else
         /*
@@ -1050,14 +1050,14 @@ static int drvHostBaseOpen(PDRVHOSTBASE pThis, PRTFILE pFileBlockDevice, PRTFILE
     if (RT_SUCCESS(rc))
     {
         rc = RTFileOpen(pFileRawDevice, pThis->pszRawDeviceOpen, fFlags);
-        if (RT_FAILURE(rc))
-        {
-            LogRel(("DVD: failed to open device %s\n", pThis->pszRawDeviceOpen));
-            RTFileClose(*pFileBlockDevice);
-        }
+        if (RT_SUCCESS(rc))
+            return rc;
+
+        LogRel(("DVD: failed to open device %s rc=%Rrc\n", pThis->pszRawDeviceOpen, rc));
+        RTFileClose(*pFileBlockDevice);
     }
     else
-        LogRel(("DVD: failed to open device %s\n", pThis->pszRawDeviceOpen));
+        LogRel(("DVD: failed to open device %s rc=%Rrc\n", pThis->pszDeviceOpen, rc));
     return rc;
 }
 #endif  /* RT_OS_SOLARIS */
@@ -1080,6 +1080,16 @@ static int drvHostBaseReopen(PDRVHOSTBASE pThis)
 
     RTFILE FileDevice;
 #ifdef RT_OS_SOLARIS
+    if (pThis->FileRawDevice != NIL_RTFILE)
+    {
+        RTFileClose(pThis->FileRawDevice);
+        pThis->FileRawDevice = NIL_RTFILE;
+    }
+    if (pThis->FileDevice != NIL_RTFILE)
+    {
+        RTFileClose(pThis->FileDevice);
+        pThis->FileDevice = NIL_RTFILE;
+    }
     RTFILE FileRawDevice;
     int rc = drvHostBaseOpen(pThis, &FileDevice, &FileRawDevice, pThis->fReadOnlyConfig);
 #else
@@ -1422,7 +1432,7 @@ int DRVHostBaseMediaPresent(PDRVHOSTBASE pThis)
         return rc;
 
     /*
-     * Determin the size.
+     * Determine the size.
      */
     uint64_t cb;
     rc = pThis->pfnGetMediaSize(pThis, &cb);
