@@ -1136,18 +1136,11 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
 
     if (RT_FAILURE(rc))
     {
-        if (VALID_PTR(pThis->pDisk))
-        {
-            VDDestroy(pThis->pDisk);
-            pThis->pDisk = NULL;
-        }
-        drvvdFreeImages(pThis);
         if (VALID_PTR(pszName))
             MMR3HeapFree(pszName);
         if (VALID_PTR(pszFormat))
             MMR3HeapFree(pszFormat);
-
-        return rc;
+        /* drvvdDestruct does the rest. */
     }
     else
     {
@@ -1193,6 +1186,8 @@ static DECLCALLBACK(int) drvvdConstruct(PPDMDRVINS pDrvIns,
         pThis->fErrorUseRuntime = true;
     }
 
+    /* else: drvvdDestruct cleans up. */
+
     LogFlow(("%s: returns %Rrc\n", __FUNCTION__, rc));
     return rc;
 }
@@ -1210,6 +1205,11 @@ static DECLCALLBACK(void) drvvdDestruct(PPDMDRVINS pDrvIns)
     PVBOXDISK pThis = PDMINS_2_DATA(pDrvIns, PVBOXDISK);
     LogFlow(("%s:\n", __FUNCTION__));
 
+    if (VALID_PTR(pThis->pDisk))
+    {
+        VDDestroy(pThis->pDisk);
+        pThis->pDisk = NULL;
+    }
     drvvdFreeImages(pThis);
 }
 
