@@ -284,6 +284,10 @@ public:
     void setMainWindow (QWidget *aMainWindow) { mMainWindow = aMainWindow; }
     QWidget *mainWindow() const { return mMainWindow; }
 
+    /* branding */
+    bool brandingIsActive (bool aForce = false);
+    QString brandingGetKey (QString aKey);
+
     bool isVMConsoleProcess() const { return !vmUuid.isNull(); }
 #ifdef VBOX_GUI_WITH_SYSTRAY
     bool isTrayMenu() const;
@@ -876,6 +880,7 @@ private:
     CVirtualBoxCallback callback;
 
     QString mVerString;
+    QString mBrandingConfig;
 
     QList <QString> mFamilyIDs;
     QList <QList <CGuestOSType> > mTypes;
