@@ -145,11 +145,15 @@ public:
     QWidget *mainWindowShown() const;
 
     /* Generic problem handlers */
+    bool askForOverridingFile (const QString& aPath, QWidget *aParent  = NULL) const;
+    bool askForOverridingFiles (const QVector<QString>& aPaths, QWidget *aParent = NULL) const;
     bool askForOverridingFileIfExists (const QString& path, QWidget *aParent = NULL) const;
-    bool askForOverridingFilesIfExists (const QStringList& aPaths, QWidget *aParent = NULL) const;
+    bool askForOverridingFilesIfExists (const QVector<QString>& aPaths, QWidget *aParent = NULL) const;
 
     void cannotDeleteFile (const QString& path, QWidget *aParent = NULL) const;
 
+    void checkForMountedWrongUSB() const;
+
     /* Special problem handlers */
     void showBETAWarning();
 
@@ -260,6 +264,7 @@ public:
                            VBoxDefs::MediaType aType, const QString &aLocation);
     void cannotCloseMedium (QWidget *aParent, const VBoxMedium &aMedium,
                             const COMResult &aResult);
+    void cannotEjectDrive();
 
     void cannotOpenSession (const CSession &session);
     void cannotOpenSession (const CVirtualBox &vbox, const CMachine &machine,
@@ -267,7 +272,6 @@ public:
 
     void cannotGetMediaAccessibility (const VBoxMedium &aMedium);
 
-#if defined Q_WS_WIN
     int confirmDeletingHostInterface (const QString &aName, QWidget *aParent = 0);
     void cannotCreateHostInterface (const CHost &aHost, QWidget *aParent = 0);
     void cannotCreateHostInterface (const CProgress &aProgress, QWidget *aParent = 0);
@@ -277,7 +281,6 @@ public:
     void cannotRemoveHostInterface (const CProgress &aProgress,
                                     const CHostNetworkInterface &aIface,
                                     QWidget *aParent = 0);
-#endif
 
     void cannotAttachUSBDevice (const CConsole &console, const QString &device);
     void cannotAttachUSBDevice (const CConsole &console, const QString &device,
@@ -345,6 +348,9 @@ public:
     void cannotImportAppliance (CAppliance *aAppliance, QWidget *aParent = NULL) const;
     void cannotImportAppliance (const CProgress &aProgress, CAppliance *aAppliance, QWidget *aParent = NULL) const;
 
+    void cannotCheckFiles (const CProgress &aProgress, QWidget *aParent = NULL) const;
+    void cannotRemoveFiles (const CProgress &aProgress, QWidget *aParent = NULL) const;
+
     void cannotExportAppliance (CAppliance *aAppliance, QWidget *aParent = NULL) const;
     void cannotExportAppliance (const CMachine &aMachine, CAppliance *aAppliance, QWidget *aParent = NULL) const;
     void cannotExportAppliance (const CProgress &aProgress, CAppliance *aAppliance, QWidget *aParent = NULL) const;
