@@ -500,7 +500,7 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent,
     /* Make non-possible to activate list elements by single click,
      * this hack should disable the current possibility to do it if present */
     if (mVMListView->style()->styleHint (QStyle::SH_ItemView_ActivateItemOnSingleClick, 0, mVMListView))
-        mVMListView->setStyleSheet ("activate-on-singleclick");
+        mVMListView->setStyleSheet ("activate-on-singleclick : 0");
 
     leftVLayout->addWidget (mVMListView);
 
@@ -1001,11 +1001,12 @@ void VBoxSelectorWnd::vmStart (const QString &aUuid /*= QUuid_null*/)
     QString env;
 #if defined (Q_WS_X11)
     /* make sure the VM process will start on the same display as the Selector */
-    {
-        const char *display = RTEnvGet ("DISPLAY");
-        if (display)
-            env.sprintf ("DISPLAY=%s", display);
-    }
+    const char *display = RTEnvGet ("DISPLAY");
+    if (display)
+        env.append(QString("DISPLAY=%1\n").arg(display));
+    const char *xauth = RTEnvGet ("XAUTHORITY");
+    if (xauth)
+        env.append(QString("XAUTHORITY=%1\n").arg(xauth));
 #endif
 
     CProgress progress = vbox.OpenRemoteSession (session, id, "GUI/Qt", env);
