@@ -36,22 +36,26 @@
 
 #ifdef Q_WS_X11
 #include <QFontDatabase>
+#include <iprt/env.h>
 #endif
 
 #include <QCleanlooksStyle>
 #include <QPlastiqueStyle>
-#include <qmessagebox.h>
-#include <qlocale.h>
-#include <qtranslator.h>
+#include <QMessageBox>
+#include <QLocale>
+#include <QTranslator>
+
+#ifdef Q_WS_X11
+# include <X11/Xlib.h>
+#endif
 
 #include <iprt/err.h>
 #include <iprt/initterm.h>
 #include <iprt/process.h>
 #include <iprt/stream.h>
+#include <VBox/err.h>
 #ifdef VBOX_WITH_HARDENING
 # include <VBox/sup.h>
-#else
-# include <VBox/err.h>
 #endif
 
 #ifdef RT_OS_LINUX
@@ -63,13 +67,7 @@
 /* XXX Temporarily. Don't rely on ther user to hack the Makefile himsef! */
 QString g_QStrHintLinuxNoMemory = QApplication::tr(
   "This error means that the kernel driver was either not able to "
-  "allocate enough memory or that some mapping operation failed.<br/><br/>"
-  "There are known problems with Linux 2.6.29. If you are running "
-  "such a kernel, please edit /usr/src/vboxdrv-*/Makefile and enable "
-  "<i>VBOX_USE_INSERT_PAGE = 1</i>. After that, re-compile the kernel "
-  "module by executing<br/><br/>"
-  "  <font color=blue>'/etc/init.d/vboxdrv setup'</font><br/><br/>"
-  "as root."
+  "allocate enough memory or that some mapping operation failed."
   );
 
 QString g_QStrHintLinuxNoDriver = QApplication::tr(
@@ -82,6 +80,22 @@ QString g_QStrHintLinuxNoDriver = QApplication::tr(
   "recompiles the vboxdrv kernel module if necessary."
   );
 
+QString g_QStrHintOtherWrongDriverVersion = QApplication::tr(
+  "The VirtualBox kernel modules do not fit to this version of "
+  "VirtualBox. The installation of VirtualBox was apparently not "
+  "successful. It may help to completely uninstall and re-install "
+  "VirtualBox."
+  );
+
+QString g_QStrHintLinuxWrongDriverVersion = QApplication::tr(
+  "The VirtualBox kernel modules do not fit to this version of "
+  "VirtualBox. The installation of VirtualBox was apparently not "
+  "successful. Executing<br/><br/>"
+  "  <font color=blue>'/etc/init.d/vboxdrv setup'</font><br/><br/>"
+  "should fix that problem. Make sure that you don't mix the "
+  "OSE version and the PUEL version of VirtualBox."
+  );
+
 QString g_QStrHintOtherNoDriver = QApplication::tr(
   "Make sure the kernel module has been loaded successfully."
   );
@@ -142,7 +156,7 @@ void bt_sighandler (int sig, siginfo_t *info, void *secret) {
 
 #endif /* DEBUG && X11 && LINUX*/
 
-#if defined(RT_OS_DARWIN) && defined(RT_ARCH_AMD64)
+#if defined(RT_OS_DARWIN)
 # include <dlfcn.h>
 # include <sys/mman.h>
 # include <iprt/asm.h>
@@ -158,7 +172,7 @@ static void ShutUpAppKit(void)
     if (!rc)
         ASMAtomicWriteU32((volatile uint32_t *)addr, 0xccc3c031); /* xor eax, eax; ret; int3 */
 }
-#endif /* DARWIN + AMD64 */
+#endif /* DARWIN */
 
 static void QtMessageOutput (QtMsgType type, const char *msg)
 {
@@ -237,7 +251,11 @@ static void showHelp()
 # ifdef VBOX_WITH_DEBUGGER_GUI
             "  --dbg                      enable the GUI debug menu\n"
             "  --debug                    like --dbg and show debug windows at VM startup\n"
+            "  --debug-command-line       like --dbg and show command line window at VM startup\n"
+            "  --debug-statistics         like --dbg and show statistics window at VM startup\n"
             "  --no-debug                 disable the GUI debug menu and debug windows\n"
+            "  --start-paused             start the VM in the paused state\n"
+            "  --start-running            start the VM running (for overriding --debug*)\n"
             "\n"
             "The following environment variables are evaluated:\n"
             "  VBOX_GUI_DBG_ENABLED       enable the GUI debug menu if set\n"
@@ -253,7 +271,7 @@ static void showHelp()
 extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
 {
     LogFlowFuncEnter();
-# if defined(RT_OS_DARWIN) && defined(RT_ARCH_AMD64)
+# if defined(RT_OS_DARWIN)
     ShutUpAppKit();
 # endif
 
@@ -306,21 +324,59 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
 
     /* scope the QIApplication variable */
     {
+#ifdef Q_WS_X11
+        /* Qt has a complex algorithm for selecting the right visual which
+         * doesn't always seem to work.  So we naively choose a visual - the
+         * default one - ourselves and pass that to Qt.  This means that we
+         * also have to open the display ourselves.
+         * We check the Qt parameter list and handle Qt's -display argument
+         * ourselves, since we open the display connection.  We also check the
+         * to see if the user has passed Qt's -visual parameter, and if so we
+         * assume that the user wants Qt to handle visual selection after all,
+         * and don't supply a visual. */
+        char *pszDisplay = NULL;
+        bool useDefaultVisual = true;
+        for (int i = 0; i < argc; ++i)
+        {
+            if (!::strcmp(argv[i], "-display") && (i + 1 < argc))
+            /* What if it isn't?  Rely on QApplication to complain? */
+            {
+                pszDisplay = argv[i + 1];
+                ++i;
+            }
+            else if (!::strcmp(argv[i], "-visual"))
+                useDefaultVisual = false;
+        }
+        Display *pDisplay = XOpenDisplay(pszDisplay);
+        if (!pDisplay)
+        {
+            RTPrintf(pszDisplay ? "Failed to open the X11 display \"%s\"!\n"
+                                : "Failed to open the X11 display!\n",
+                     pszDisplay);
+            return 0;
+        }
+        Visual *pVisual =   useDefaultVisual
+                          ? DefaultVisual(pDisplay, DefaultScreen(pDisplay))
+                          : NULL;
+        /* Now create the application object */
+        QIApplication a (pDisplay, argc, argv, (Qt::HANDLE) pVisual);
+#else /* Q_WS_X11 */
         QIApplication a (argc, argv);
+#endif /* Q_WS_X11 */
 
         /* Qt4.3 version has the QProcess bug which freezing the application
          * for 30 seconds. This bug is internally used at initialization of
          * Cleanlooks style. So we have to change this style to another one.
          * See http://trolltech.com/developer/task-tracker/index_html?id=179200&method=entry
          * for details. */
-        if (QString (qVersion()).startsWith ("4.3") &&
+        if (VBoxGlobal::qtRTVersionString().startsWith ("4.3") &&
             qobject_cast <QCleanlooksStyle*> (QApplication::style()))
             QApplication::setStyle (new QPlastiqueStyle);
 
 #ifdef Q_OS_SOLARIS
         /* Solaris have some issue with cleanlooks style which leads to application
          * crash in case of using it on Qt4.4 version, lets make the same substitute */
-        if (QString (qVersion()).startsWith ("4.4") &&
+        if (VBoxGlobal::qtRTVersionString().startsWith ("4.4") &&
             qobject_cast <QCleanlooksStyle*> (QApplication::style()))
             QApplication::setStyle (new QPlastiqueStyle);
 #endif
@@ -376,24 +432,13 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
 
 #ifdef Q_WS_X11
         /* version check (major.minor are sensitive, fix number is ignored) */
-        QString ver_str = QString::fromLatin1 (QT_VERSION_STR);
-        QString ver_str_base = ver_str.section ('.', 0, 1);
-        QString rt_ver_str = QString::fromLatin1 (qVersion());
-        uint ver =
-            (ver_str.section ('.', 0, 0).toInt() << 16) +
-            (ver_str.section ('.', 1, 1).toInt() << 8) +
-            ver_str.section ('.', 2, 2).toInt();
-        uint rt_ver =
-            (rt_ver_str.section ('.', 0, 0).toInt() << 16) +
-            (rt_ver_str.section ('.', 1, 1).toInt() << 8) +
-            rt_ver_str.section ('.', 2, 2).toInt();
-        if (rt_ver < (ver & 0xFFFF00))
+        if (VBoxGlobal::qtRTVersion() < (VBoxGlobal::qtCTVersion() & 0xFFFF00))
         {
             QString msg =
                 QApplication::tr ("Executable <b>%1</b> requires Qt %2.x, found Qt %3.")
                                   .arg (qAppName())
-                                  .arg (ver_str_base)
-                                  .arg (rt_ver_str);
+                                  .arg (VBoxGlobal::qtCTVersionString().section ('.', 0, 1))
+                                  .arg (VBoxGlobal::qtRTVersionString());
             QMessageBox::critical (
                 0, QApplication::tr ("Incompatible Qt Library Error"),
                 msg, QMessageBox::Abort, 0);
@@ -426,6 +471,8 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
 #endif
 #endif
 
+            vboxProblem().checkForMountedWrongUSB();
+
             VBoxGlobalSettings settings = vboxGlobal().settings();
             /* Process known keys */
             bool noSelector = settings.isFeatureActive ("noSelector");
@@ -448,10 +495,12 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char ** /*envp*/)
             }
             else
             {
+#ifndef DEBUG
                 /* Check for BETA version */
                 QString vboxVersion (vboxGlobal().virtualBox().GetVersion());
                 if (vboxVersion.contains ("BETA"))
                     vboxProblem().showBETAWarning();
+#endif
 
                 vboxGlobal().setMainWindow (&vboxGlobal().selectorWnd());
 #ifdef VBOX_GUI_WITH_SYSTRAY
@@ -507,8 +556,11 @@ int main (int argc, char **argv, char **envp)
     bool fInitSUPLib = false;
     for (int i = 1; i < argc; i++)
     {
-        if (    !::strcmp(argv[i], "--startvm")
-            ||  !::strcmp(argv[i], "-startvm"))
+        /* NOTE: the check here must match the corresponding check for the
+         * options to start a VM in hardenedmain.cpp and VBoxGlobal.cpp exactly,
+         * otherwise there will be weird error messages. */
+        if (   !::strcmp(argv[i], "--startvm")
+            || !::strcmp(argv[i], "-startvm"))
         {
             fInitSUPLib = true;
             break;
@@ -542,11 +594,16 @@ int main (int argc, char **argv, char **envp)
                 msgText += g_QStrHintLinuxNoMemory;
                 break;
 # endif
-# if 0 /** @todo Enable after 2.2.0 (/ NLS unfreeze). */
             case VERR_VM_DRIVER_NOT_ACCESSIBLE:
                 msgText += QApplication::tr ("Kernel driver not accessible");
                 break;
-# endif
+            case VERR_VM_DRIVER_VERSION_MISMATCH:
+# ifdef RT_OS_LINUX
+                msgText += g_QStrHintLinuxWrongDriverVersion;
+# else
+                msgText += g_QStrHintOtherWrongDriverVersion;
+# endif 
+                break;
             default:
                 msgText += QApplication::tr (
                         "Unknown %2 error during initialization of the Runtime"
@@ -576,7 +633,7 @@ int main (int argc, char **argv, char **envp)
  */
 extern "C" DECLEXPORT(void) TrustedError (const char *pszWhere, SUPINITOP enmWhat, int rc, const char *pszMsgFmt, va_list va)
 {
-# if defined(RT_OS_DARWIN) && defined(RT_ARCH_AMD64)
+# if defined(RT_OS_DARWIN)
     ShutUpAppKit();
 # endif
 
@@ -613,6 +670,13 @@ extern "C" DECLEXPORT(void) TrustedError (const char *pszWhere, SUPINITOP enmWha
                 msgText += g_QStrHintLinuxNoMemory;
             else
 # endif
+            if (rc == VERR_VM_DRIVER_VERSION_MISMATCH)
+# ifdef RT_OS_LINUX
+                msgText += g_QStrHintLinuxWrongDriverVersion;
+# else
+                msgText += g_QStrHintOtherWrongDriverVersion;
+# endif
+            else
                 msgText += g_QStrHintReinstall;
             break;
         case kSupInitOp_Integrity:
