@@ -54,6 +54,7 @@
 #include <iprt/semaphore.h>
 #include <iprt/spinlock.h>
 #include <iprt/mp.h>
+#include <iprt/power.h>
 #include <iprt/process.h>
 #include <iprt/thread.h>
 #include <iprt/initterm.h>
@@ -244,7 +245,7 @@ int _init(void)
         }
         else
             LogRel((DEVICE_NAME ":VBoxDrvSolarisAttach: supdrvInitDevExt failed\n"));
-        RTR0Term();
+        RTR0TermForced();
     }
     else
         LogRel((DEVICE_NAME ":VBoxDrvSolarisAttach: failed to init R0Drv\n"));
@@ -271,7 +272,7 @@ int _fini(void)
     AssertRC(rc);
     g_Spinlock = NIL_RTSPINLOCK;
 
-    RTR0Term();
+    RTR0TermForced();
 
     memset(&g_DevExt, 0, sizeof(g_DevExt));
 
@@ -318,6 +319,14 @@ static int VBoxDrvSolarisAttach(dev_info_t *pDip, ddi_attach_cmd_t enmCmd)
             pState = ddi_get_soft_state(g_pVBoxDrvSolarisState, instance);
 #endif
 
+            /*
+             * Register for suspend/resume notifications
+             */
+            rc = ddi_prop_create(DDI_DEV_T_NONE, pDip, DDI_PROP_CANSLEEP /* kmem alloc can sleep */,
+                                "pm-hardware-state", "needs-suspend-resume", sizeof("needs-suspend-resume"));
+            if (rc != DDI_PROP_SUCCESS)
+                LogRel((DEVICE_NAME ":Suspend/Resume notification registeration failed.\n"));
+
             /*
              * Register ourselves as a character device, pseudo-driver
              */
@@ -342,11 +351,15 @@ static int VBoxDrvSolarisAttach(dev_info_t *pDip, ddi_attach_cmd_t enmCmd)
 
         case DDI_RESUME:
         {
+#if 0
             RTSemFastMutexRequest(g_DevExt.mtxGip);
             if (g_DevExt.pGipTimer)
                 RTTimerStart(g_DevExt.pGipTimer, 0);
 
             RTSemFastMutexRelease(g_DevExt.mtxGip);
+#endif
+            RTPowerSignalEvent(RTPOWEREVENT_RESUME);
+            LogFlow((DEVICE_NAME ": Awakened from suspend.\n"));
             return DDI_SUCCESS;
         }
 
@@ -383,17 +396,23 @@ static int VBoxDrvSolarisDetach(dev_info_t *pDip, ddi_detach_cmd_t enmCmd)
             ddi_remove_minor_node(pDip, NULL);
             ddi_soft_state_free(g_pVBoxDrvSolarisState, instance);
 #endif
+            ddi_prop_remove_all(pDip);
             return DDI_SUCCESS;
         }
 
         case DDI_SUSPEND:
         {
+#if 0
             RTSemFastMutexRequest(g_DevExt.mtxGip);
             if (g_DevExt.pGipTimer && g_DevExt.cGipUsers > 0)
                 RTTimerStop(g_DevExt.pGipTimer);
 
             RTSemFastMutexRelease(g_DevExt.mtxGip);
+#endif
+            RTPowerSignalEvent(RTPOWEREVENT_SUSPEND);
+            LogFlow((DEVICE_NAME ": Falling to suspend mode.\n"));
             return DDI_SUCCESS;
+
         }
 
         default:
