@@ -118,8 +118,6 @@ static void crServerTearDown( void )
     crSPUUnloadChain(cr_server.head_spu);
     cr_server.head_spu = NULL;
 #endif
-
-    crUnloadOpenGL();
 }
 
 static void crServerClose( unsigned int id )
@@ -320,10 +318,16 @@ GLboolean crVBoxServerInit(void)
     return GL_TRUE;
 }
 
-void crVBoxServerAddClient(uint32_t u32ClientID)
+int32_t crVBoxServerAddClient(uint32_t u32ClientID)
 {
-    CRClient *newClient = (CRClient *) crCalloc(sizeof(CRClient));
-    
+    CRClient *newClient;
+
+    if (cr_server.numClients>=CR_MAX_CLIENTS)
+    {
+        return VERR_MAX_THRDS_REACHED;
+    }
+
+    newClient = (CRClient *) crCalloc(sizeof(CRClient));    
     crDebug("crServer: AddClient u32ClientID=%d", u32ClientID);
 
     newClient->spu_id = 0;
@@ -337,6 +341,8 @@ void crVBoxServerAddClient(uint32_t u32ClientID)
     cr_server.clients[cr_server.numClients++] = newClient;
 
     crServerAddToRunQueue(newClient);
+
+    return VINF_SUCCESS;
 }
 
 void crVBoxServerRemoveClient(uint32_t u32ClientID)
@@ -364,7 +370,7 @@ void crVBoxServerRemoveClient(uint32_t u32ClientID)
     crServerDeleteClient(pClient);
 }
 
-void crVBoxServerClientWrite(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t cbBuffer)
+int32_t crVBoxServerClientWrite(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t cbBuffer)
 {
     CRClient *pClient;
     int32_t i;
@@ -382,6 +388,8 @@ void crVBoxServerClientWrite(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t cb
     pClient = cr_server.clients[i];
     CRASSERT(pClient);
 
+    if (!pClient->conn->vMajor) return VERR_NOT_SUPPORTED;
+
     CRASSERT(pBuffer);
 
     /* This should never fire unless we start to multithread */
@@ -437,6 +445,8 @@ void crVBoxServerClientWrite(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t cb
     crStateResetCurrentPointers(&cr_server.current);
 
     CRASSERT(!pClient->conn->allow_redir_ptr || crNetNumMessages(pClient->conn)==0);
+
+    return VINF_SUCCESS;
 }
 
 int32_t crVBoxServerClientRead(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t *pcbBuffer)
@@ -457,9 +467,11 @@ int32_t crVBoxServerClientRead(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t
     pClient = cr_server.clients[i];
     CRASSERT(pClient);
 
+    if (!pClient->conn->vMajor) return VERR_NOT_SUPPORTED;
+
     if (pClient->conn->cbHostBuffer > *pcbBuffer)
     {
-        crWarning("crServer: [%lx] ClientRead u32ClientID=%d FAIL, host buffer too small %d of %d",
+        crDebug("crServer: [%lx] ClientRead u32ClientID=%d FAIL, host buffer too small %d of %d",
                   crThreadID(), u32ClientID, *pcbBuffer, pClient->conn->cbHostBuffer);
 
         /* Return the size of needed buffer */
@@ -481,6 +493,33 @@ int32_t crVBoxServerClientRead(uint32_t u32ClientID, uint8_t *pBuffer, uint32_t
     return VINF_SUCCESS;
 }
 
+int32_t crVBoxServerClientSetVersion(uint32_t u32ClientID, uint32_t vMajor, uint32_t vMinor)
+{
+    CRClient *pClient;
+    int32_t i;
+
+    for (i = 0; i < cr_server.numClients; i++)
+    {
+        if (cr_server.clients[i] && cr_server.clients[i]->conn 
+            && cr_server.clients[i]->conn->u32ClientID==u32ClientID) 
+        {
+            break;
+        }
+    }
+    pClient = cr_server.clients[i];
+    CRASSERT(pClient);
+
+    pClient->conn->vMajor = vMajor;
+    pClient->conn->vMinor = vMinor;
+
+    if (vMajor != CR_PROTOCOL_VERSION_MAJOR
+        || vMinor != CR_PROTOCOL_VERSION_MINOR)
+    {
+        return VERR_NOT_SUPPORTED;
+    }
+    else return VINF_SUCCESS;
+}
+
 int
 CRServerMain(int argc, char *argv[])
 {
