@@ -19,10 +19,6 @@
  * additional information or have any questions.
  */
 
-#if defined(RT_OS_WINDOWS)
-#elif defined(RT_OS_LINUX)
-#endif
-
 #ifdef VBOX_WITH_SYS_V_IPC_SESSION_WATCHER
 #   include <errno.h>
 #   include <sys/types.h>
@@ -258,6 +254,7 @@ STDMETHODIMP Session::GetPID (ULONG *aPid)
 
 STDMETHODIMP Session::GetRemoteConsole (IConsole **aConsole)
 {
+    LogFlowThisFuncEnter();
     AssertReturn (aConsole, E_POINTER);
 
     AutoCaller autoCaller (this);
@@ -277,6 +274,8 @@ STDMETHODIMP Session::GetRemoteConsole (IConsole **aConsole)
 
     mConsole.queryInterfaceTo (aConsole);
 
+    LogFlowThisFuncLeave();
+
     return S_OK;
 }
 
@@ -642,7 +641,7 @@ STDMETHODIMP Session::OnUSBDeviceAttach (IUSBDevice *aDevice,
     return mConsole->onUSBDeviceAttach (aDevice, aError, aMaskedIfs);
 }
 
-STDMETHODIMP Session::OnUSBDeviceDetach (IN_GUID aId,
+STDMETHODIMP Session::OnUSBDeviceDetach (IN_BSTR aId,
                                          IVirtualBoxErrorInfo *aError)
 {
     LogFlowThisFunc (("\n"));
@@ -805,7 +804,7 @@ HRESULT Session::close (bool aFinalRelease, bool aFromServer)
     }
 
     /* go to the closing state */
-    mState =  SessionState_Closing;
+    mState = SessionState_Closing;
 
     if (mType == SessionType_Direct)
     {
