@@ -138,30 +138,33 @@ public:
     /* IVirtualBox methods */
 
     STDMETHOD(CreateMachine) (IN_BSTR aName, IN_BSTR aOsTypeId, IN_BSTR aBaseFolder,
-                              IN_GUID aId, IMachine **aMachine);
+                              IN_BSTR aId, IMachine **aMachine);
     STDMETHOD(CreateLegacyMachine) (IN_BSTR aName, IN_BSTR aOsTypeId, IN_BSTR aSettingsFile,
-                                    IN_GUID aId, IMachine **aMachine);
+                                    IN_BSTR aId, IMachine **aMachine);
     STDMETHOD(OpenMachine) (IN_BSTR aSettingsFile, IMachine **aMachine);
     STDMETHOD(RegisterMachine) (IMachine *aMachine);
-    STDMETHOD(GetMachine) (IN_GUID aId, IMachine **aMachine);
+    STDMETHOD(GetMachine) (IN_BSTR aId, IMachine **aMachine);
     STDMETHOD(FindMachine) (IN_BSTR aName, IMachine **aMachine);
-    STDMETHOD(UnregisterMachine) (IN_GUID aId, IMachine **aMachine);
+    STDMETHOD(UnregisterMachine) (IN_BSTR aId, IMachine **aMachine);
     STDMETHOD(CreateAppliance) (IAppliance **anAppliance);
 
     STDMETHOD(CreateHardDisk)(IN_BSTR aFormat, IN_BSTR aLocation,
                                IHardDisk **aHardDisk);
-    STDMETHOD(OpenHardDisk) (IN_BSTR aLocation, AccessMode_T accessMode, IHardDisk **aHardDisk);
-    STDMETHOD(GetHardDisk) (IN_GUID aId, IHardDisk **aHardDisk);
+    STDMETHOD(OpenHardDisk) (IN_BSTR aLocation, AccessMode_T accessMode,
+                             BOOL aSetImageId, IN_BSTR aImageId,
+                             BOOL aSetParentId, IN_BSTR aParentId,
+                             IHardDisk **aHardDisk);
+    STDMETHOD(GetHardDisk) (IN_BSTR aId, IHardDisk **aHardDisk);
     STDMETHOD(FindHardDisk) (IN_BSTR aLocation, IHardDisk **aHardDisk);
 
-    STDMETHOD(OpenDVDImage) (IN_BSTR aLocation, IN_GUID aId,
+    STDMETHOD(OpenDVDImage) (IN_BSTR aLocation, IN_BSTR aId,
                              IDVDImage **aDVDImage);
-    STDMETHOD(GetDVDImage) (IN_GUID aId, IDVDImage **aDVDImage);
+    STDMETHOD(GetDVDImage) (IN_BSTR aId, IDVDImage **aDVDImage);
     STDMETHOD(FindDVDImage) (IN_BSTR aLocation, IDVDImage **aDVDImage);
 
-    STDMETHOD(OpenFloppyImage) (IN_BSTR aLocation, IN_GUID aId,
+    STDMETHOD(OpenFloppyImage) (IN_BSTR aLocation, IN_BSTR aId,
                                 IFloppyImage **aFloppyImage);
-    STDMETHOD(GetFloppyImage) (IN_GUID aId, IFloppyImage **aFloppyImage);
+    STDMETHOD(GetFloppyImage) (IN_BSTR aId, IFloppyImage **aFloppyImage);
     STDMETHOD(FindFloppyImage) (IN_BSTR aLocation, IFloppyImage **aFloppyImage);
 
     STDMETHOD(GetGuestOSType) (IN_BSTR aId, IGuestOSType **aType);
@@ -170,11 +173,11 @@ public:
     STDMETHOD(GetNextExtraDataKey) (IN_BSTR aKey, BSTR *aNextKey, BSTR *aNextValue);
     STDMETHOD(GetExtraData) (IN_BSTR aKey, BSTR *aValue);
     STDMETHOD(SetExtraData) (IN_BSTR aKey, IN_BSTR aValue);
-    STDMETHOD(OpenSession) (ISession *aSession, IN_GUID aMachineId);
-    STDMETHOD(OpenRemoteSession) (ISession *aSession, IN_GUID aMachineId,
+    STDMETHOD(OpenSession) (ISession *aSession, IN_BSTR aMachineId);
+    STDMETHOD(OpenRemoteSession) (ISession *aSession, IN_BSTR aMachineId,
                                   IN_BSTR aType, IN_BSTR aEnvironment,
                                   IProgress **aProgress);
-    STDMETHOD(OpenExistingSession) (ISession *aSession, IN_GUID aMachineId);
+    STDMETHOD(OpenExistingSession) (ISession *aSession, IN_BSTR aMachineId);
 
     STDMETHOD(RegisterCallback) (IVirtualBoxCallback *aCallback);
     STDMETHOD(UnregisterCallback) (IVirtualBoxCallback *aCallback);
@@ -358,6 +361,16 @@ public:
      */
     RWLockHandle *hardDiskTreeLockHandle() { return &mHardDiskTreeLockHandle; }
 
+    /**
+     * Reimplements VirtualBoxWithTypedChildren::childrenLock() to return a
+     * dedicated lock instead of the main object lock. The dedicated lock for
+     * child map operations frees callers of init() methods of these children
+     * from acquiring a write parent (VirtualBox) lock (which would be mandatory
+     * otherwise). Since VirtualBox has a lot of heterogenous children which
+     * init() methods are called here and there, it definitely makes sense.
+     */
+    RWLockHandle *childrenLock() { return &mChildrenMapLockHandle; }
+
     /* for VirtualBoxSupportErrorInfoImpl */
     static const wchar_t *getComponentName() { return L"VirtualBox"; }
 
@@ -376,16 +389,6 @@ private:
 
     typedef std::map <Guid, ComObjPtr<HardDisk> > HardDiskMap;
 
-    /**
-     * Reimplements VirtualBoxWithTypedChildren::childrenLock() to return a
-     * dedicated lock instead of the main object lock. The dedicated lock for
-     * child map operations frees callers of init() methods of these children
-     * from acquiring a write parent (VirtualBox) lock (which would be mandatory
-     * otherwise). Since VirtualBox has a lot of heterogenous children which
-     * init() methods are called here and there, it definitely makes sense.
-     */
-    RWLockHandle *childrenLock() { return &mChildrenMapLockHandle; }
-
     HRESULT checkMediaForConflicts2 (const Guid &aId, const Bstr &aLocation,
                                      Utf8Str &aConflictType);
 
@@ -421,6 +424,8 @@ private:
 
             const Bstr mName;
             RTFILE mHandle;
+            /** Flag indicating that the config file is read-only. */
+            BOOL mReadonly;
         };
 
         // const data members not requiring locking
@@ -551,6 +556,5 @@ private:
      */
     ComObjPtr <VirtualBox, ComWeakRef> mVirtualBox;
 };
-
 #endif // ____H_VIRTUALBOXIMPL
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
