@@ -98,7 +98,7 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
             }
             else
             {
-                free(pMemFreeBSD->Core.pv, M_IPRTMOBJ);
+                contigfree(pMemFreeBSD->Core.pv, pMemFreeBSD->Core.cb, M_IPRTMOBJ);
                 if (pMemFreeBSD->pMappingObject)
                 {
                     rc = vm_map_remove(kernel_map,
@@ -111,13 +111,21 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
 
         case RTR0MEMOBJTYPE_LOCK:
         {
+            int fFlags = VM_MAP_WIRE_NOHOLES;
             vm_map_t pMap = kernel_map;
+
             if (pMemFreeBSD->Core.u.Lock.R0Process != NIL_RTR0PROCESS)
+            {
                 pMap = &((struct proc *)pMemFreeBSD->Core.u.Lock.R0Process)->p_vmspace->vm_map;
+                fFlags |= VM_MAP_WIRE_USER;
+            }
+            else
+                fFlags |= VM_MAP_WIRE_SYSTEM;
+
             rc = vm_map_unwire(pMap,
                                (vm_offset_t)pMemFreeBSD->Core.pv,
                                (vm_offset_t)pMemFreeBSD->Core.pv + pMemFreeBSD->Core.cb,
-                               VM_MAP_WIRE_SYSTEM | VM_MAP_WIRE_NOHOLES);
+                               fFlags);
             AssertMsg(rc == KERN_SUCCESS, ("%#x", rc));
             break;
         }
@@ -136,7 +144,18 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
 
         case RTR0MEMOBJTYPE_MAPPING:
         {
-            /** @todo Figure out mapping... */
+            vm_map_t pMap = kernel_map;
+
+            /* vm_map_remove will unmap the pages we inserted with pmap_enter */
+            AssertMsg(pMemFreeBSD->pMappingObject != NULL, ("MappingObject is NULL\n"));
+            if (pMemFreeBSD->Core.u.Mapping.R0Process != NIL_RTR0PROCESS)
+                pMap = &((struct proc *)pMemFreeBSD->Core.u.Mapping.R0Process)->p_vmspace->vm_map;
+
+            rc = vm_map_remove(pMap,
+                               (vm_offset_t)pMemFreeBSD->Core.pv,
+                               (vm_offset_t)pMemFreeBSD->Core.pv + pMemFreeBSD->Core.cb);
+            AssertMsg(rc == KERN_SUCCESS, ("%#x", rc));
+            break;
         }
 
         /* unused: */
@@ -148,8 +167,6 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
             return VERR_INTERNAL_ERROR;
     }
 
-    Assert(!pMemFreeBSD->pMappingObject);
-
     return VINF_SUCCESS;
 }
 
@@ -157,29 +174,14 @@ int rtR0MemObjNativeFree(RTR0MEMOBJ pMem)
 int rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecutable)
 {
     int rc;
+    size_t cPages = cb >> PAGE_SHIFT;
 
     /* create the object. */
     PRTR0MEMOBJFREEBSD pMemFreeBSD = (PRTR0MEMOBJFREEBSD)rtR0MemObjNew(sizeof(*pMemFreeBSD), RTR0MEMOBJTYPE_PAGE, NULL, cb);
     if (!pMemFreeBSD)
         return VERR_NO_MEMORY;
 
-    /*
-     * We've two options here both expressed nicely by how kld allocates
-     * memory for the module bits:
-     *      http://fxr.watson.org/fxr/source/kern/link_elf.c?v=RELENG62#L701
-     */
-#if 0
-    pMemFreeBSD->Core.pv = malloc(cb, M_IPRTMOBJ, M_ZERO);
-    if (pMemFreeBSD->Core.pv)
-    {
-        *ppMem = &pMemFreeBSD->Core;
-        return VINF_SUCCESS;
-    }
-    rc = VERR_NO_MEMORY;
-    NOREF(fExecutable);
-
-#else
-    pMemFreeBSD->pObject = vm_object_allocate(OBJT_DEFAULT, cb >> PAGE_SHIFT);
+    pMemFreeBSD->pObject = vm_object_allocate(OBJT_DEFAULT, cPages);
     if (pMemFreeBSD->pObject)
     {
         vm_offset_t MapAddress = vm_map_min(kernel_map);
@@ -196,28 +198,67 @@ int rtR0MemObjNativeAllocPage(PPRTR0MEMOBJINTERNAL ppMem, size_t cb, bool fExecu
                          FALSE);                        /* cow (copy-on-write) */
         if (rc == KERN_SUCCESS)
         {
-            rc = vm_map_wire(kernel_map,                /* map */
-                             MapAddress,                /* start */
-                             MapAddress + cb,           /* end */
-                             VM_MAP_WIRE_SYSTEM | VM_MAP_WIRE_NOHOLES);
-            if (rc == KERN_SUCCESS)
+            vm_offset_t AddressDst = MapAddress;
+
+            rc = VINF_SUCCESS;
+
+            VM_OBJECT_LOCK(pMemFreeBSD->pObject);
+            for (size_t iPage = 0; iPage < cPages; iPage++)
+            {
+                vm_pindex_t PageIndex = OFF_TO_IDX(AddressDst);
+                vm_page_t   pPage;
+
+                pPage = vm_page_alloc(pMemFreeBSD->pObject, PageIndex,
+                                      VM_ALLOC_NOBUSY | VM_ALLOC_SYSTEM |
+                                      VM_ALLOC_WIRED);
+                if (pPage)
+                {
+                    vm_page_lock_queues();
+                    vm_page_wire(pPage);
+                    vm_page_unlock_queues();
+                    /* Put the page into the page table now. */
+#if __FreeBSD_version >= 701105
+                    pmap_enter(kernel_map->pmap, AddressDst, VM_PROT_NONE, pPage,
+                               fExecutable
+                               ? VM_PROT_ALL
+                               : VM_PROT_RW,
+                               TRUE);
+#else
+                    pmap_enter(kernel_map->pmap, AddressDst, pPage,
+                               fExecutable
+                               ? VM_PROT_ALL
+                               : VM_PROT_RW,
+                               TRUE);
+#endif
+                }
+                else
+                {
+                    /*
+                     * Allocation failed. vm_map_remove will remove any
+                     * page already alocated.
+                     */
+                    rc = VERR_NO_MEMORY;
+                    break;
+                }
+                AddressDst += PAGE_SIZE;
+            }
+            VM_OBJECT_UNLOCK(pMemFreeBSD->pObject);
+
+            if (rc == VINF_SUCCESS)
             {
                 pMemFreeBSD->Core.pv = (void *)MapAddress;
                 *ppMem = &pMemFreeBSD->Core;
                 return VINF_SUCCESS;
             }
 
-           vm_map_remove(kernel_map,
-                         MapAddress,
-                         MapAddress + cb);
+            vm_map_remove(kernel_map,
+                          MapAddress,
+                          MapAddress + cb);
         }
-        else
-            vm_object_deallocate(pMemFreeBSD->pObject);
         rc = VERR_NO_MEMORY; /** @todo fix translation (borrow from darwin) */
     }
     else
         rc = VERR_NO_MEMORY;
-#endif
 
     rtR0MemObjDelete(&pMemFreeBSD->Core);
     return rc;
@@ -329,9 +370,10 @@ int rtR0MemObjNativeEnterPhys(PPRTR0MEMOBJINTERNAL ppMem, RTHCPHYS Phys, size_t
 }
 
 
-int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t cb, RTR0PROCESS R0Process)
+int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t cb, uint32_t fAccess, RTR0PROCESS R0Process)
 {
     int rc;
+    NOREF(fAccess);
 
     /* create the object. */
     PRTR0MEMOBJFREEBSD pMemFreeBSD = (PRTR0MEMOBJFREEBSD)rtR0MemObjNew(sizeof(*pMemFreeBSD), RTR0MEMOBJTYPE_LOCK, (void *)R3Ptr, cb);
@@ -343,9 +385,9 @@ int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t c
      * resource usage restrictions, so we'll call vm_map_wire directly.
      */
     rc = vm_map_wire(&((struct proc *)R0Process)->p_vmspace->vm_map, /* the map */
-                     (vm_offset_t)R3Ptr,                            /* start */
-                     (vm_offset_t)R3Ptr + cb,                       /* end */
-                     VM_MAP_WIRE_SYSTEM | VM_MAP_WIRE_NOHOLES);     /* flags - SYSTEM? */
+                     (vm_offset_t)R3Ptr,                             /* start */
+                     (vm_offset_t)R3Ptr + cb,                        /* end */
+                     VM_MAP_WIRE_USER | VM_MAP_WIRE_NOHOLES);        /* flags */
     if (rc == KERN_SUCCESS)
     {
         pMemFreeBSD->Core.u.Lock.R0Process = R0Process;
@@ -357,9 +399,10 @@ int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t c
 }
 
 
-int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb)
+int rtR0MemObjNativeLockKernel(PPRTR0MEMOBJINTERNAL ppMem, void *pv, size_t cb, uint32_t fAccess)
 {
     int rc;
+    NOREF(fAccess);
 
     /* create the object. */
     PRTR0MEMOBJFREEBSD pMemFreeBSD = (PRTR0MEMOBJFREEBSD)rtR0MemObjNew(sizeof(*pMemFreeBSD), RTR0MEMOBJTYPE_LOCK, pv, cb);
@@ -414,7 +457,7 @@ static int rtR0MemObjNativeReserveInMap(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixe
     {
         vm_offset_t MapAddress = pvFixed != (void *)-1
                                ? (vm_offset_t)pvFixed
-                               : vm_map_min(kernel_map);
+                               : vm_map_min(pMap);
         if (pvFixed)
             vm_map_remove(pMap,
                           MapAddress,
@@ -428,7 +471,7 @@ static int rtR0MemObjNativeReserveInMap(PPRTR0MEMOBJINTERNAL ppMem, void *pvFixe
                          pvFixed == (void *)-1,         /* find_space */
                          VM_PROT_NONE,                  /* protection */
                          VM_PROT_ALL,                   /* max(_prot) ?? */
-                         FALSE);                        /* cow (copy-on-write) */
+                         0);                            /* cow (copy-on-write) */
         if (rc == KERN_SUCCESS)
         {
             if (R0Process != NIL_RTR0PROCESS)
@@ -543,91 +586,110 @@ int rtR0MemObjNativeMapKernel(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ pMemToMap,
 }
 
 
+/* see http://markmail.org/message/udhq33tefgtyfozs */
 int rtR0MemObjNativeMapUser(PPRTR0MEMOBJINTERNAL ppMem, RTR0MEMOBJ pMemToMap, RTR3PTR R3PtrFixed, size_t uAlignment, unsigned fProt, RTR0PROCESS R0Process)
 {
     AssertMsgReturn(R0Process == RTR0ProcHandleSelf(), ("%p != %p\n", R0Process, RTR0ProcHandleSelf()), VERR_NOT_SUPPORTED);
     AssertMsgReturn(R3PtrFixed == (RTR3PTR)-1, ("%p\n", R3PtrFixed), VERR_NOT_SUPPORTED);
 
-#if 0
-    int rc;
-    void *pvR0;
-    void *pvR3 = NULL;
-    PRTR0MEMOBJFREEBSD pMemToMapOs2 = (PRTR0MEMOBJFREEBSD)pMemToMap;
-    switch (pMemToMapOs2->Core.enmType)
+    int             rc;
+    vm_object_t     pObjectToMap = ((PRTR0MEMOBJFREEBSD)pMemToMap)->pObject;
+    struct proc    *pProc     = (struct proc *)R0Process;
+    struct vm_map  *pProcMap  = &pProc->p_vmspace->vm_map;
+
+    /* calc protection */
+    vm_prot_t       ProtectionFlags = 0;
+    if ((fProt & RTMEM_PROT_NONE) == RTMEM_PROT_NONE)
+        ProtectionFlags = VM_PROT_NONE;
+    if ((fProt & RTMEM_PROT_READ) == RTMEM_PROT_READ)
+        ProtectionFlags |= VM_PROT_READ;
+    if ((fProt & RTMEM_PROT_WRITE) == RTMEM_PROT_WRITE)
+        ProtectionFlags |= VM_PROT_WRITE;
+    if ((fProt & RTMEM_PROT_EXEC) == RTMEM_PROT_EXEC)
+        ProtectionFlags |= VM_PROT_EXECUTE;
+
+    /* calc mapping address */
+    PROC_LOCK(pProc);
+    vm_offset_t AddrR3 = round_page((vm_offset_t)pProc->p_vmspace->vm_daddr + lim_max(pProc, RLIMIT_DATA));
+    PROC_UNLOCK(pProc);
+
+    vm_object_t pObjectNew = vm_object_allocate(OBJT_PHYS, pMemToMap->cb >> PAGE_SHIFT);
+    if (!RT_UNLIKELY(pObjectNew))
+        return VERR_NO_MEMORY;
+
+    /* Insert the object in the map. */
+    rc = vm_map_find(pProcMap,              /* Map to insert the object in */
+                        pObjectNew  ,          /* Object to map */
+                        0,                     /* Start offset in the object */
+                        &AddrR3,               /* Start address IN/OUT */
+                        pMemToMap->cb,         /* Size of the mapping */
+                        TRUE,                  /* Whether a suitable address should be searched for first */
+                        ProtectionFlags,       /* protection flags */
+                        VM_PROT_ALL,           /* Maximum protection flags */
+                        0);                    /* Copy on write */
+
+    /* Map the memory page by page into the destination map. */
+    if (rc == KERN_SUCCESS)
+    {
+        size_t         cLeft        = pMemToMap->cb >> PAGE_SHIFT;
+        vm_offset_t    AddrToMap    = (vm_offset_t)pMemToMap->pv;
+        pmap_t         pPhysicalMap = pProcMap->pmap;
+        vm_offset_t    AddrR3Dst    = AddrR3;
+
+        /* Insert the memory page by page into the mapping. */
+        while (cLeft-- > 0)
+        {
+            vm_page_t Page = PHYS_TO_VM_PAGE(vtophys(AddrToMap));
+
+#if __FreeBSD_version >= 701105
+            pmap_enter(pPhysicalMap, AddrR3Dst, VM_PROT_NONE, Page, ProtectionFlags, TRUE);
+#else
+            pmap_enter(pPhysicalMap, AddrR3Dst, Page, ProtectionFlags, TRUE);
+#endif
+            AddrToMap += PAGE_SIZE;
+            AddrR3Dst += PAGE_SIZE;
+        }
+        pObjectToMap = pObjectNew;
+    }
+    else
+        vm_object_deallocate(pObjectNew);
+
+    if (rc == KERN_SUCCESS)
     {
         /*
-         * These has kernel mappings.
+         * Create a mapping object for it.
          */
-        case RTR0MEMOBJTYPE_PAGE:
-        case RTR0MEMOBJTYPE_LOW:
-        case RTR0MEMOBJTYPE_CONT:
-            pvR0 = pMemToMapOs2->Core.pv;
-            break;
+        PRTR0MEMOBJFREEBSD pMemFreeBSD = (PRTR0MEMOBJFREEBSD)rtR0MemObjNew(sizeof(RTR0MEMOBJFREEBSD),
+                                                                           RTR0MEMOBJTYPE_MAPPING,
+                                                                           (void *)AddrR3,
+                                                                           pMemToMap->cb);
+        if (pMemFreeBSD)
+        {
+            Assert((vm_offset_t)pMemFreeBSD->Core.pv == AddrR3);
+            pMemFreeBSD->Core.u.Mapping.R0Process = R0Process;
+            pMemFreeBSD->pMappingObject = pObjectToMap;
+            *ppMem = &pMemFreeBSD->Core;
+            return VINF_SUCCESS;
+        }
 
-        case RTR0MEMOBJTYPE_PHYS:
-            pvR0 = pMemToMapOs2->Core.pv;
-#if 0/* this is wrong. */
-            if (!pvR0)
-            {
-                /* no ring-0 mapping, so allocate a mapping in the process. */
-                AssertMsgReturn(uAlignment == PAGE_SIZE, ("%#zx\n", uAlignment), VERR_NOT_SUPPORTED);
-                AssertMsgReturn(fProt & RTMEM_PROT_WRITE, ("%#x\n", fProt), VERR_NOT_SUPPORTED);
-                Assert(!pMemToMapOs2->Core.u.Phys.fAllocated);
-                ULONG ulPhys = pMemToMapOs2->Core.u.Phys.PhysBase;
-                rc = KernVMAlloc(pMemToMapOs2->Core.cb, VMDHA_PHYS | VMDHA_PROCESS, &pvR3, (PPVOID)&ulPhys, NULL);
-                if (rc)
-                    return RTErrConvertFromOS2(rc);
-            }
-            break;
-#endif
-            return VERR_NOT_SUPPORTED;
-
-        case RTR0MEMOBJTYPE_LOCK:
-            if (pMemToMapOs2->Core.u.Lock.R0Process != NIL_RTR0PROCESS)
-                return VERR_NOT_SUPPORTED; /** @todo implement this... */
-            pvR0 = pMemToMapOs2->Core.pv;
-            break;
-
-        case RTR0MEMOBJTYPE_PHYS_NC:
-        case RTR0MEMOBJTYPE_RES_VIRT:
-        case RTR0MEMOBJTYPE_MAPPING:
-        default:
-            AssertMsgFailed(("enmType=%d\n", pMemToMapOs2->Core.enmType));
-            return VERR_INTERNAL_ERROR;
+        rc = vm_map_remove(pProcMap, ((vm_offset_t)AddrR3), ((vm_offset_t)AddrR3) + pMemToMap->cb);
+        AssertMsg(rc == KERN_SUCCESS, ("Deleting mapping failed\n"));
     }
 
-    /*
-     * Map the ring-0 memory into the current process.
-     */
-    if (!pvR3)
-    {
-        Assert(pvR0);
-        ULONG flFlags = 0;
-        if (uAlignment == PAGE_SIZE)
-            flFlags |= VMDHGP_4MB;
-        if (fProt & RTMEM_PROT_WRITE)
-            flFlags |= VMDHGP_WRITE;
-        rc = RTR0Os2DHVMGlobalToProcess(flFlags, pvR0, pMemToMapOs2->Core.cb, &pvR3);
-        if (rc)
-            return RTErrConvertFromOS2(rc);
-    }
-    Assert(pvR3);
+    if (pObjectToMap)
+        vm_object_deallocate(pObjectToMap);
 
-    /*
-     * Create a mapping object for it.
-     */
-    PRTR0MEMOBJFREEBSD pMemFreeBSD = (PRTR0MEMOBJFREEBSD)rtR0MemObjNew(RT_OFFSETOF(RTR0MEMOBJOS2, Lock), RTR0MEMOBJTYPE_MAPPING, pvR3, pMemToMapOs2->Core.cb);
-    if (pMemFreeBSD)
-    {
-        Assert(pMemFreeBSD->Core.pv == pvR3);
-        pMemFreeBSD->Core.u.Mapping.R0Process = R0Process;
-        *ppMem = &pMemFreeBSD->Core;
-        return VINF_SUCCESS;
-    }
-    KernVMFree(pvR3);
     return VERR_NO_MEMORY;
-#endif
-    return VERR_NOT_IMPLEMENTED;
+}
+
+
+int rtR0MemObjNativeProtect(PRTR0MEMOBJINTERNAL pMem, size_t offSub, size_t cbSub, uint32_t fProt)
+{
+    NOREF(pMem);
+    NOREF(offSub);
+    NOREF(cbSub);
+    NOREF(fProt);
+    return VERR_NOT_SUPPORTED;
 }
 
 
@@ -638,17 +700,17 @@ RTHCPHYS rtR0MemObjNativeGetPagePhysAddr(PRTR0MEMOBJINTERNAL pMem, size_t iPage)
     switch (pMemFreeBSD->Core.enmType)
     {
         case RTR0MEMOBJTYPE_LOCK:
-        {
             if (    pMemFreeBSD->Core.u.Lock.R0Process != NIL_RTR0PROCESS
                 &&  pMemFreeBSD->Core.u.Lock.R0Process != (RTR0PROCESS)curproc)
             {
                 /* later */
                 return NIL_RTHCPHYS;
             }
-        }
+            /* fall thru*/
         case RTR0MEMOBJTYPE_PAGE:
+        case RTR0MEMOBJTYPE_MAPPING:
         {
-            uint8_t *pb = (uint8_t *)pMemFreeBSD->Core.pv + ((size_t)iPage << PAGE_SHIFT);
+            uint8_t *pb = (uint8_t *)pMemFreeBSD->Core.pv + (iPage << PAGE_SHIFT);
             return vtophys(pb);
         }
 
@@ -660,9 +722,9 @@ RTHCPHYS rtR0MemObjNativeGetPagePhysAddr(PRTR0MEMOBJINTERNAL pMem, size_t iPage)
 
         case RTR0MEMOBJTYPE_PHYS_NC:
         case RTR0MEMOBJTYPE_RES_VIRT:
-        case RTR0MEMOBJTYPE_MAPPING:
         case RTR0MEMOBJTYPE_LOW:
         default:
             return NIL_RTHCPHYS;
     }
 }
+
