@@ -36,21 +36,22 @@
 #include <VBox/log.h>
 #include <VBox/selm.h>
 #include <VBox/iom.h>
-#include <iprt/param.h>
 #include <iprt/assert.h>
 #include <iprt/asm.h>
-#include <iprt/string.h>
-#include <iprt/memobj.h>
 #include <iprt/cpuset.h>
+#include <iprt/memobj.h>
+#include <iprt/param.h>
 #include <iprt/power.h>
+#include <iprt/string.h>
+#include <iprt/thread.h>
 #include "HWVMXR0.h"
 #include "HWSVMR0.h"
 
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
-static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2);
-static DECLCALLBACK(void) hwaccmR0DisableCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2);
+static DECLCALLBACK(void) hwaccmR0EnableCpuCallback(RTCPUID idCpu, void *pvUser1, void *pvUser2);
+static DECLCALLBACK(void) hwaccmR0DisableCpuCallback(RTCPUID idCpu, void *pvUser1, void *pvUser2);
 static DECLCALLBACK(void) HWACCMR0InitCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2);
 static              int   hwaccmR0CheckCpuRcArray(int *paRc, unsigned cErrorCodes, RTCPUID *pidCpu);
 static DECLCALLBACK(void) hwaccmR0PowerCallback(RTPOWEREVENT enmEvent, void *pvUser);
@@ -131,6 +132,7 @@ static struct
 
     HWACCMSTATE                     enmHwAccmState;
 
+    bool                            fGlobalInit;
     volatile        bool            fSuspended;
 } HWACCMR0Globals;
 
@@ -162,6 +164,9 @@ VMMR0DECL(int) HWACCMR0Init(void)
     HWACCMR0Globals.pfnTermVM           = HWACCMR0DummyTermVM;
     HWACCMR0Globals.pfnSetupVM          = HWACCMR0DummySetupVM;
 
+    /* Default is global VT-x/AMD-V init */
+    HWACCMR0Globals.fGlobalInit         = true;
+
     /*
      * Check for VT-x and AMD-V capabilities
      */
@@ -453,6 +458,7 @@ VMMR0DECL(int) HWACCMR0Term(void)
     if (   HWACCMR0Globals.vmx.fSupported
         && HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
     {
+        Assert(HWACCMR0Globals.fGlobalInit);
         rc = SUPR0EnableVTx(false /* fEnable */);
         for (unsigned iCpu = 0; iCpu < RT_ELEMENTS(HWACCMR0Globals.aCpuInfo); iCpu++)
         {
@@ -462,23 +468,32 @@ VMMR0DECL(int) HWACCMR0Term(void)
     }
     else
     {
-        int aRc[RTCPUSET_MAX_CPUS];
-
         Assert(!HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx);
         if (!HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
         {
             rc = RTPowerNotificationDeregister(hwaccmR0PowerCallback, 0);
             Assert(RT_SUCCESS(rc));
         }
+        else
+            rc = VINF_SUCCESS;
 
-        memset(aRc, 0, sizeof(aRc));
-        rc = RTMpOnAll(hwaccmR0DisableCPU, aRc, NULL);
-        Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+        /* Only disable VT-x/AMD-V on all CPUs if we enabled it before. */
+        if (HWACCMR0Globals.fGlobalInit)
+        {
+            int aRc[RTCPUSET_MAX_CPUS];
+
+            memset(aRc, 0, sizeof(aRc));
+            rc = RTMpOnAll(hwaccmR0DisableCpuCallback, aRc, NULL);
+            Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+#ifdef VBOX_STRICT
+            for (unsigned i=0;i<RT_ELEMENTS(HWACCMR0Globals.aCpuInfo);i++)
+                AssertMsgRC(aRc[i], ("hwaccmR0DisableCpuCallback failed for cpu %d with rc=%d\n", i, aRc[i]));
+#endif
+        }
 
         /* Free the per-cpu pages used for VT-x and AMD-V */
         for (unsigned i=0;i<RT_ELEMENTS(HWACCMR0Globals.aCpuInfo);i++)
         {
-            AssertMsgRC(aRc[i], ("hwaccmR0DisableCPU failed for cpu %d with rc=%d\n", i, aRc[i]));
             if (HWACCMR0Globals.aCpuInfo[i].pMemObj != NIL_RTR0MEMOBJ)
             {
                 RTR0MemObjFree(HWACCMR0Globals.aCpuInfo[i].pMemObj, false);
@@ -504,7 +519,7 @@ static DECLCALLBACK(void) HWACCMR0InitCPU(RTCPUID idCpu, void *pvUser1, void *pv
     int     *paRc         = (int *)pvUser2;
     uint64_t val;
 
-#ifdef LOG_ENABLED
+#if defined(LOG_ENABLED) && !defined(DEBUG_bird)
     SUPR0Printf("HWACCMR0InitCPU cpu %d\n", idCpu);
 #endif
     Assert(idCpu == (RTCPUID)RTMpCpuIdToSetIndex(idCpu)); /// @todo fix idCpu == index assumption (rainy day)
@@ -582,6 +597,8 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
     {
         int rc;
 
+        HWACCMR0Globals.fGlobalInit = pVM->hwaccm.s.fGlobalInit;
+
         if (   HWACCMR0Globals.vmx.fSupported
             && HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
         {
@@ -593,6 +610,8 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
                     HWACCMR0Globals.aCpuInfo[iCpu].fConfigured = true;
                     Assert(HWACCMR0Globals.aCpuInfo[iCpu].pMemObj == NIL_RTR0MEMOBJ);
                 }
+                /* If the host provides a VT-x init API, then we'll rely on that for global init. */
+                HWACCMR0Globals.fGlobalInit = pVM->hwaccm.s.fGlobalInit = true;
             }
             else
                 AssertMsgFailed(("HWACCMR0EnableAllCpus/SUPR0EnableVTx: rc=%Rrc\n", rc));
@@ -621,18 +640,23 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
                     Assert(pvR0);
                     ASMMemZeroPage(pvR0);
 
-#ifdef LOG_ENABLED
+#if defined(LOG_ENABLED) && !defined(DEBUG_bird)
                     SUPR0Printf("address %x phys %x\n", pvR0, (uint32_t)RTR0MemObjGetPagePhysAddr(HWACCMR0Globals.aCpuInfo[i].pMemObj, 0));
 #endif
                 }
             }
-            /* First time, so initialize each cpu/core */
-            rc = RTMpOnAll(hwaccmR0EnableCPU, (void *)pVM, aRc);
+            if (HWACCMR0Globals.fGlobalInit)
+            {
+                /* First time, so initialize each cpu/core */
+                rc = RTMpOnAll(hwaccmR0EnableCpuCallback, (void *)pVM, aRc);
 
-            /* Check the return code of all invocations. */
-            if (RT_SUCCESS(rc))
-                rc = hwaccmR0CheckCpuRcArray(aRc, RT_ELEMENTS(aRc), &idCpu);
-            AssertMsgRC(rc, ("HWACCMR0EnableAllCpus failed for cpu %d with rc=%d\n", idCpu, rc));
+                /* Check the return code of all invocations. */
+                if (RT_SUCCESS(rc))
+                    rc = hwaccmR0CheckCpuRcArray(aRc, RT_ELEMENTS(aRc), &idCpu);
+                AssertMsgRC(rc, ("HWACCMR0EnableAllCpus failed for cpu %d with rc=%d\n", idCpu, rc));
+            }
+            else
+                rc = VINF_SUCCESS;
         }
 
         return rc;
@@ -641,17 +665,14 @@ VMMR0DECL(int) HWACCMR0EnableAllCpus(PVM pVM)
 }
 
 /**
- * Worker function passed to RTMpOnAll, RTMpOnOthers and RTMpOnSpecific that
- * is to be called on the target cpus.
+ * Disable VT-x or AMD-V on the current CPU
  *
+ * @returns VBox status code.
+ * @param   pVM         VM handle (can be 0!)
  * @param   idCpu       The identifier for the CPU the function is called on.
- * @param   pvUser1     The 1st user argument.
- * @param   pvUser2     The 2nd user argument.
  */
-static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+static int hwaccmR0EnableCpu(PVM pVM, RTCPUID idCpu)
 {
-    PVM             pVM = (PVM)pvUser1;     /* can be NULL! */
-    int            *paRc = (int *)pvUser2;
     void           *pvPageCpu;
     RTHCPHYS        pPageCpuPhys;
     PHWACCM_CPUINFO pCpu = &HWACCMR0Globals.aCpuInfo[idCpu];
@@ -660,7 +681,7 @@ static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *
     Assert(idCpu == (RTCPUID)RTMpCpuIdToSetIndex(idCpu)); /// @todo fix idCpu == index assumption (rainy day)
     Assert(idCpu < RT_ELEMENTS(HWACCMR0Globals.aCpuInfo));
     Assert(!pCpu->fConfigured);
-    Assert(ASMAtomicReadBool(&pCpu->fInUse) == false);
+    Assert(!HWACCMR0Globals.fGlobalInit || ASMAtomicReadBool(&pCpu->fInUse) == false);
 
     pCpu->idCpu     = idCpu;
 
@@ -674,19 +695,81 @@ static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *
     if (!pCpu->pMemObj)
     {
         AssertFailed();
-        paRc[idCpu] = VERR_INTERNAL_ERROR;
-        return;
+        return VERR_INTERNAL_ERROR;
     }
 
     pvPageCpu    = RTR0MemObjAddress(pCpu->pMemObj);
     pPageCpuPhys = RTR0MemObjGetPagePhysAddr(pCpu->pMemObj, 0);
 
-    paRc[idCpu]  = HWACCMR0Globals.pfnEnableCpu(pCpu, pVM, pvPageCpu, pPageCpuPhys);
-    AssertRC(paRc[idCpu]);
-    if (RT_SUCCESS(paRc[idCpu]))
+    int rc  = HWACCMR0Globals.pfnEnableCpu(pCpu, pVM, pvPageCpu, pPageCpuPhys);
+    AssertRC(rc);
+    if (RT_SUCCESS(rc))
         pCpu->fConfigured = true;
 
-    return;
+    return rc;
+}
+
+
+/**
+ * Worker function passed to RTMpOnAll, RTMpOnOthers and RTMpOnSpecific that
+ * is to be called on the target cpus.
+ *
+ * @param   idCpu       The identifier for the CPU the function is called on.
+ * @param   pvUser1     The 1st user argument.
+ * @param   pvUser2     The 2nd user argument.
+ */
+static DECLCALLBACK(void) hwaccmR0EnableCpuCallback(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+{
+    PVM             pVM = (PVM)pvUser1;     /* can be NULL! */
+    int            *paRc = (int *)pvUser2;
+
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        paRc[idCpu] = VINF_SUCCESS;
+        AssertFailed();
+        return;
+    }
+
+    paRc[idCpu] = hwaccmR0EnableCpu(pVM, idCpu);
+}
+
+
+/**
+ * Disable VT-x or AMD-V on the current CPU
+ *
+ * @returns VBox status code.
+ * @param   idCpu       The identifier for the CPU the function is called on.
+ */
+static int hwaccmR0DisableCpu(RTCPUID idCpu)
+{
+    void           *pvPageCpu;
+    RTHCPHYS        pPageCpuPhys;
+    int             rc;
+    PHWACCM_CPUINFO pCpu = &HWACCMR0Globals.aCpuInfo[idCpu];
+
+    Assert(!HWACCMR0Globals.vmx.fSupported || !HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx);
+    Assert(idCpu == (RTCPUID)RTMpCpuIdToSetIndex(idCpu)); /// @todo fix idCpu == index assumption (rainy day)
+    Assert(idCpu < RT_ELEMENTS(HWACCMR0Globals.aCpuInfo));
+    Assert(!HWACCMR0Globals.fGlobalInit || ASMAtomicReadBool(&pCpu->fInUse) == false);
+    Assert(!pCpu->fConfigured || pCpu->pMemObj);
+
+    if (!pCpu->pMemObj)
+        return (pCpu->fConfigured) ? VERR_NO_MEMORY : VINF_SUCCESS /* not initialized. */;
+
+    pvPageCpu    = RTR0MemObjAddress(pCpu->pMemObj);
+    pPageCpuPhys = RTR0MemObjGetPagePhysAddr(pCpu->pMemObj, 0);
+
+    if (pCpu->fConfigured)
+    {
+        rc = HWACCMR0Globals.pfnDisableCpu(pCpu, pvPageCpu, pPageCpuPhys);
+        AssertRC(rc);
+        pCpu->fConfigured = false;
+    }
+    else
+        rc = VINF_SUCCESS; /* nothing to do */
+
+    pCpu->uCurrentASID = 0;
+    return rc;
 }
 
 /**
@@ -697,36 +780,18 @@ static DECLCALLBACK(void) hwaccmR0EnableCPU(RTCPUID idCpu, void *pvUser1, void *
  * @param   pvUser1     The 1st user argument.
  * @param   pvUser2     The 2nd user argument.
  */
-static DECLCALLBACK(void) hwaccmR0DisableCPU(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+static DECLCALLBACK(void) hwaccmR0DisableCpuCallback(RTCPUID idCpu, void *pvUser1, void *pvUser2)
 {
-    void           *pvPageCpu;
-    RTHCPHYS        pPageCpuPhys;
     int            *paRc = (int *)pvUser1;
-    PHWACCM_CPUINFO pCpu = &HWACCMR0Globals.aCpuInfo[idCpu];
 
-    Assert(!HWACCMR0Globals.vmx.fSupported || !HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx);
-    Assert(idCpu == (RTCPUID)RTMpCpuIdToSetIndex(idCpu)); /// @todo fix idCpu == index assumption (rainy day)
-    Assert(idCpu < RT_ELEMENTS(HWACCMR0Globals.aCpuInfo));
-    Assert(ASMAtomicReadBool(&pCpu->fInUse) == false);
-    Assert(!pCpu->fConfigured || pCpu->pMemObj);
-
-    if (!pCpu->pMemObj)
-        return;
-
-    pvPageCpu    = RTR0MemObjAddress(pCpu->pMemObj);
-    pPageCpuPhys = RTR0MemObjGetPagePhysAddr(pCpu->pMemObj, 0);
-
-    if (pCpu->fConfigured)
+    if (!HWACCMR0Globals.fGlobalInit)
     {
-        paRc[idCpu] = HWACCMR0Globals.pfnDisableCpu(pCpu, pvPageCpu, pPageCpuPhys);
-        AssertRC(paRc[idCpu]);
-        pCpu->fConfigured = false;
+        paRc[idCpu] = VINF_SUCCESS;
+        AssertFailed();
+        return;
     }
-    else
-        paRc[idCpu] = VINF_SUCCESS; /* nothing to do */
 
-    pCpu->uCurrentASID = 0;
-    return;
+    paRc[idCpu] = hwaccmR0DisableCpu(idCpu);
 }
 
 /**
@@ -759,13 +824,17 @@ static DECLCALLBACK(void) hwaccmR0PowerCallback(RTPOWEREVENT enmEvent, void *pvU
         memset(aRc, 0, sizeof(aRc));
         if (enmEvent == RTPOWEREVENT_SUSPEND)
         {
-            /* Turn off VT-x or AMD-V on all CPUs. */
-            rc = RTMpOnAll(hwaccmR0DisableCPU, aRc, NULL);
-            Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+            if (HWACCMR0Globals.fGlobalInit)
+            {
+                /* Turn off VT-x or AMD-V on all CPUs. */
+                rc = RTMpOnAll(hwaccmR0DisableCpuCallback, aRc, NULL);
+                Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+            }
+            /* else nothing to do here for the local init case */
         }
         else
         {
-            /* Reinit the CPUs from scratch as the suspend state has messed with the MSRs. */
+            /* Reinit the CPUs from scratch as the suspend state might have messed with the MSRs. (lousy BIOSes as usual) */
             rc = RTMpOnAll(HWACCMR0InitCPU, (void *)((HWACCMR0Globals.vmx.fSupported) ? X86_CPUID_VENDOR_INTEL_EBX : X86_CPUID_VENDOR_AMD_EBX), aRc);
             Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
 
@@ -776,9 +845,13 @@ static DECLCALLBACK(void) hwaccmR0PowerCallback(RTPOWEREVENT enmEvent, void *pvU
                 SUPR0Printf("hwaccmR0PowerCallback HWACCMR0InitCPU failed with %d\n", rc);
 #endif
 
-            /* Turn VT-x or AMD-V back on on all CPUs. */
-            rc = RTMpOnAll(hwaccmR0EnableCPU, NULL, aRc);
-            Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+            if (HWACCMR0Globals.fGlobalInit)
+            {
+                /* Turn VT-x or AMD-V back on on all CPUs. */
+                rc = RTMpOnAll(hwaccmR0EnableCpuCallback, NULL, aRc);
+                Assert(RT_SUCCESS(rc) || rc == VERR_NOT_SUPPORTED);
+            }
+            /* else nothing to do here for the local init case */
         }
     }
     if (enmEvent == RTPOWEREVENT_RESUME)
@@ -835,7 +908,17 @@ VMMR0DECL(int) HWACCMR0InitVM(PVM pVM)
 
     pVM->hwaccm.s.uMaxASID                  = HWACCMR0Globals.uMaxASID;
 
-    for (unsigned i=0;i<pVM->cCPUs;i++)
+
+    if (!pVM->hwaccm.s.cMaxResumeLoops) /* allow ring-3 overrides */
+    {
+        pVM->hwaccm.s.cMaxResumeLoops       = 1024;
+#ifdef VBOX_WITH_VMMR0_DISABLE_PREEMPTION
+        if (RTThreadPreemptIsPendingTrusty())
+            pVM->hwaccm.s.cMaxResumeLoops   = 8192;
+#endif
+    }
+
+    for (VMCPUID i = 0; i < pVM->cCPUs; i++)
     {
         PVMCPU pVCpu = &pVM->aCpus[i];
 
@@ -851,7 +934,7 @@ VMMR0DECL(int) HWACCMR0InitVM(PVM pVM)
     RTCCUINTREG     fFlags = ASMIntDisableFlags();
     PHWACCM_CPUINFO pCpu = HWACCMR0GetCurrentCpu();
 
-    /* @note Not correct as we can be rescheduled to a different cpu, but the fInUse case is mostly for debugging. */
+    /* Note: Not correct as we can be rescheduled to a different cpu, but the fInUse case is mostly for debugging. */
     ASMAtomicWriteBool(&pCpu->fInUse, true);
     ASMSetFlags(fFlags);
 
@@ -920,15 +1003,29 @@ VMMR0DECL(int) HWACCMR0SetupVM(PVM pVM)
 
     ASMAtomicWriteBool(&pCpu->fInUse, true);
 
-    for (unsigned i=0;i<pVM->cCPUs;i++)
+    for (VMCPUID i = 0; i < pVM->cCPUs; i++)
     {
         /* On first entry we'll sync everything. */
         pVM->aCpus[i].hwaccm.s.fContextUseFlags = HWACCM_CHANGED_ALL;
     }
 
+    /* Enable VT-x or AMD-V if local init is required. */
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        rc = hwaccmR0EnableCpu(pVM, idCpu);
+        AssertRCReturn(rc, rc);
+    }
+
     /* Setup VT-x or AMD-V. */
     rc = HWACCMR0Globals.pfnSetupVM(pVM);
 
+    /* Disable VT-x or AMD-V if local init was done before. */
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        rc = hwaccmR0DisableCpu(idCpu);
+        AssertRC(rc);
+    }
+
     ASMAtomicWriteBool(&pCpu->fInUse, false);
 
     return rc;
@@ -953,13 +1050,16 @@ VMMR0DECL(int) HWACCMR0Enter(PVM pVM, PVMCPU pVCpu)
     AssertReturn(!ASMAtomicReadBool(&HWACCMR0Globals.fSuspended), VERR_HWACCM_SUSPEND_PENDING);
     ASMAtomicWriteBool(&pCpu->fInUse, true);
 
-    pCtx = CPUMQueryGuestCtxPtrEx(pVM, pVCpu);
+    AssertMsg(pVCpu->hwaccm.s.idEnteredCpu == NIL_RTCPUID, ("%d", (int)pVCpu->hwaccm.s.idEnteredCpu));
+    pVCpu->hwaccm.s.idEnteredCpu = idCpu;
+
+    pCtx = CPUMQueryGuestCtxPtr(pVCpu);
 
     /* Always load the guest's FPU/XMM state on-demand. */
-    CPUMDeactivateGuestFPUState(pVM);
+    CPUMDeactivateGuestFPUState(pVCpu);
 
     /* Always load the guest's debug state on-demand. */
-    CPUMDeactivateGuestDebugState(pVM);
+    CPUMDeactivateGuestDebugState(pVCpu);
 
     /* Always reload the host context and the guest's CR0 register. (!!!!) */
     pVCpu->hwaccm.s.fContextUseFlags |= HWACCM_CHANGED_GUEST_CR0 | HWACCM_CHANGED_HOST_CONTEXT;
@@ -970,6 +1070,13 @@ VMMR0DECL(int) HWACCMR0Enter(PVM pVM, PVMCPU pVCpu)
     else
         pVM->hwaccm.s.u64RegisterMask = UINT64_C(0xFFFFFFFF);
 
+    /* Enable VT-x or AMD-V if local init is required. */
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        rc = hwaccmR0EnableCpu(pVM, idCpu);
+        AssertRCReturn(rc, rc);
+    }
+
     rc  = HWACCMR0Globals.pfnEnterSession(pVM, pVCpu, pCpu);
     AssertRC(rc);
     /* We must save the host context here (VT-x) as we might be rescheduled on a different cpu after a long jump back to ring 3. */
@@ -981,13 +1088,12 @@ VMMR0DECL(int) HWACCMR0Enter(PVM pVM, PVMCPU pVCpu)
     /* keep track of the CPU owning the VMCS for debugging scheduling weirdness and ring-3 calls. */
     if (RT_SUCCESS(rc))
     {
-        AssertMsg(pVCpu->hwaccm.s.idEnteredCpu == NIL_RTCPUID, ("%d", (int)pVCpu->hwaccm.s.idEnteredCpu));
-        pVCpu->hwaccm.s.idEnteredCpu = idCpu;
-
 #ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
         PGMDynMapMigrateAutoSet(pVCpu);
 #endif
     }
+    else
+        pVCpu->hwaccm.s.idEnteredCpu = NIL_RTCPUID;
     return rc;
 }
 
@@ -1008,7 +1114,7 @@ VMMR0DECL(int) HWACCMR0Leave(PVM pVM, PVMCPU pVCpu)
 
     AssertReturn(!ASMAtomicReadBool(&HWACCMR0Globals.fSuspended), VERR_HWACCM_SUSPEND_PENDING);
 
-    pCtx = CPUMQueryGuestCtxPtrEx(pVM, pVCpu);
+    pCtx = CPUMQueryGuestCtxPtr(pVCpu);
 
     /* Note:  It's rather tricky with longjmps done by e.g. Log statements or the page fault handler.
      *        We must restore the host FPU here to make absolutely sure we don't leave the guest FPU state active
@@ -1037,6 +1143,18 @@ VMMR0DECL(int) HWACCMR0Leave(PVM pVM, PVMCPU pVCpu)
 #endif
     pVCpu->hwaccm.s.idEnteredCpu = NIL_RTCPUID;
 
+    /* Disable VT-x or AMD-V if local init was done before. */
+    if (!HWACCMR0Globals.fGlobalInit)
+    {
+        rc = hwaccmR0DisableCpu(idCpu);
+        AssertRC(rc);
+
+        /* Reset these to force a TLB flush for the next entry. (-> EXPENSIVE) */
+        pVCpu->hwaccm.s.idLastCpu    = NIL_RTCPUID;
+        pVCpu->hwaccm.s.uCurrentASID = 0;
+        VMCPU_FF_SET(pVCpu, VMCPU_FF_TLB_FLUSH);
+    }
+
     ASMAtomicWriteBool(&pCpu->fInUse, false);
     return rc;
 }
@@ -1051,13 +1169,13 @@ VMMR0DECL(int) HWACCMR0Leave(PVM pVM, PVMCPU pVCpu)
 VMMR0DECL(int) HWACCMR0RunGuestCode(PVM pVM, PVMCPU pVCpu)
 {
     CPUMCTX *pCtx;
-    RTCPUID  idCpu = RTMpCpuId(); NOREF(idCpu);
     int      rc;
 #ifdef VBOX_STRICT
+    RTCPUID  idCpu = RTMpCpuId(); NOREF(idCpu);
     PHWACCM_CPUINFO pCpu = &HWACCMR0Globals.aCpuInfo[idCpu];
 #endif
 
-    Assert(!VM_FF_ISPENDING(pVM, VM_FF_PGM_SYNC_CR3 | VM_FF_PGM_SYNC_CR3_NON_GLOBAL));
+    Assert(!VMCPU_FF_ISPENDING(pVCpu, VMCPU_FF_PGM_SYNC_CR3 | VMCPU_FF_PGM_SYNC_CR3_NON_GLOBAL));
     Assert(HWACCMR0Globals.aCpuInfo[idCpu].fConfigured);
     AssertReturn(!ASMAtomicReadBool(&HWACCMR0Globals.fSuspended), VERR_HWACCM_SUSPEND_PENDING);
     Assert(ASMAtomicReadBool(&pCpu->fInUse) == true);
@@ -1066,7 +1184,7 @@ VMMR0DECL(int) HWACCMR0RunGuestCode(PVM pVM, PVMCPU pVCpu)
     PGMDynMapStartAutoSet(pVCpu);
 #endif
 
-    pCtx = CPUMQueryGuestCtxPtrEx(pVM, pVCpu);
+    pCtx = CPUMQueryGuestCtxPtr(pVCpu);
 
     rc = HWACCMR0Globals.pfnRunGuestCode(pVM, pVCpu, pCtx);
 
@@ -1123,9 +1241,9 @@ VMMR0DECL(int)   HWACCMR0TestSwitcher3264(PVM pVM)
     uint32_t aParam[5] = {0, 1, 2, 3, 4};
     int      rc;
 
-    pCtx = CPUMQueryGuestCtxPtrEx(pVM, pVCpu);
+    pCtx = CPUMQueryGuestCtxPtr(pVCpu);
 
-    STAM_PROFILE_ADV_START(&pVCpu->hwaccm.s.StatWorldSwitch3264, z);   
+    STAM_PROFILE_ADV_START(&pVCpu->hwaccm.s.StatWorldSwitch3264, z);
     if (pVM->hwaccm.s.vmx.fSupported)
         rc = VMXR0Execute64BitsHandler(pVM, pVCpu, pCtx, pVM->hwaccm.s.pfnTest64, 5, &aParam[0]);
     else
@@ -1171,6 +1289,82 @@ VMMR0DECL(PHWACCM_CPUINFO) HWACCMR0GetCurrentCpuEx(RTCPUID idCpu)
     return &HWACCMR0Globals.aCpuInfo[idCpu];
 }
 
+/**
+ * Returns the VMCPU of the current EMT thread.
+ *
+ * @param   pVM         The VM to operate on.
+ */
+VMMR0DECL(PVMCPU)  HWACCMR0GetVMCPU(PVM pVM)
+{
+    /* RTMpCpuId had better be cheap. */
+    RTCPUID idHostCpu = RTMpCpuId();
+
+    /** @todo optimize for large number of VCPUs when that becomes more common. */
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCPUs; idCpu++)
+    {
+        PVMCPU pVCpu = &pVM->aCpus[idCpu];
+
+        if (pVCpu->hwaccm.s.idEnteredCpu == idHostCpu)
+            return pVCpu;
+    }
+    return NULL;
+}
+
+/**
+ * Returns the VMCPU id of the current EMT thread.
+ *
+ * @param   pVM         The VM to operate on.
+ */
+VMMR0DECL(VMCPUID) HWACCMR0GetVMCPUId(PVM pVM)
+{
+    PVMCPU pVCpu = HWACCMR0GetVMCPU(pVM);
+    if (pVCpu)
+        return pVCpu->idCpu;
+
+    return 0;
+}
+
+/**
+ * Save a pending IO read.
+ *
+ * @param   pVCpu           The VMCPU to operate on.
+ * @param   GCPtrRip        Address of IO instruction
+ * @param   GCPtrRipNext    Address of the next instruction
+ * @param   uPort           Port address
+ * @param   uAndVal         And mask for saving the result in eax
+ * @param   cbSize          Read size
+ */
+VMMR0DECL(void) HWACCMR0SavePendingIOPortRead(PVMCPU pVCpu, RTGCPTR GCPtrRip, RTGCPTR GCPtrRipNext, unsigned uPort, unsigned uAndVal, unsigned cbSize)
+{
+    pVCpu->hwaccm.s.PendingIO.enmType         = HWACCMPENDINGIO_PORT_READ;
+    pVCpu->hwaccm.s.PendingIO.GCPtrRip        = GCPtrRip;
+    pVCpu->hwaccm.s.PendingIO.GCPtrRipNext    = GCPtrRipNext;
+    pVCpu->hwaccm.s.PendingIO.s.Port.uPort    = uPort;
+    pVCpu->hwaccm.s.PendingIO.s.Port.uAndVal  = uAndVal;
+    pVCpu->hwaccm.s.PendingIO.s.Port.cbSize   = cbSize;
+    return;
+}
+
+/**
+ * Save a pending IO write.
+ *
+ * @param   pVCpu           The VMCPU to operate on.
+ * @param   GCPtrRIP        Address of IO instruction
+ * @param   uPort           Port address
+ * @param   uAndVal         And mask for fetching the result from eax
+ * @param   cbSize          Read size
+ */
+VMMR0DECL(void) HWACCMR0SavePendingIOPortWrite(PVMCPU pVCpu, RTGCPTR GCPtrRip, RTGCPTR GCPtrRipNext, unsigned uPort, unsigned uAndVal, unsigned cbSize)
+{
+    pVCpu->hwaccm.s.PendingIO.enmType         = HWACCMPENDINGIO_PORT_WRITE;
+    pVCpu->hwaccm.s.PendingIO.GCPtrRip        = GCPtrRip;
+    pVCpu->hwaccm.s.PendingIO.GCPtrRipNext    = GCPtrRipNext;
+    pVCpu->hwaccm.s.PendingIO.s.Port.uPort    = uPort;
+    pVCpu->hwaccm.s.PendingIO.s.Port.uAndVal  = uAndVal;
+    pVCpu->hwaccm.s.PendingIO.s.Port.cbSize   = cbSize;
+    return;
+}
+
 /**
  * Disable VT-x if it's active *and* the current switcher turns off paging
  *
@@ -1185,7 +1379,8 @@ VMMR0DECL(int) HWACCMR0EnterSwitcher(PVM pVM, bool *pfVTxDisabled)
     *pfVTxDisabled = false;
 
     if (    HWACCMR0Globals.enmHwAccmState != HWACCMSTATE_ENABLED
-        ||  !HWACCMR0Globals.vmx.fSupported /* no such issues with AMD-V */)
+        ||  !HWACCMR0Globals.vmx.fSupported /* no such issues with AMD-V */
+        ||  !pVM->hwaccm.s.fGlobalInit      /* Local init implies the CPU is currently not in VMX root mode. */)
         return VINF_SUCCESS;    /* nothing to do */
 
     switch(VMMGetSwitcher(pVM))
@@ -1194,7 +1389,7 @@ VMMR0DECL(int) HWACCMR0EnterSwitcher(PVM pVM, bool *pfVTxDisabled)
     case VMMSWITCHER_PAE_TO_PAE:
         return VINF_SUCCESS;    /* safe switchers as they don't turn off paging */
 
-    case VMMSWITCHER_32_TO_PAE: 
+    case VMMSWITCHER_32_TO_PAE:
     case VMMSWITCHER_PAE_TO_32: /* is this one actually used?? */
     case VMMSWITCHER_AMD64_TO_32:
     case VMMSWITCHER_AMD64_TO_PAE:
@@ -1232,7 +1427,8 @@ VMMR0DECL(int) HWACCMR0LeaveSwitcher(PVM pVM, bool fVTxDisabled)
         return VINF_SUCCESS;    /* nothing to do */
 
     Assert(   HWACCMR0Globals.enmHwAccmState == HWACCMSTATE_ENABLED
-           && HWACCMR0Globals.vmx.fSupported);
+           && HWACCMR0Globals.vmx.fSupported
+           && pVM->hwaccm.s.fGlobalInit);
 
     PHWACCM_CPUINFO pCpu = HWACCMR0GetCurrentCpu();
     void           *pvPageCpu;
@@ -1246,7 +1442,6 @@ VMMR0DECL(int) HWACCMR0LeaveSwitcher(PVM pVM, bool fVTxDisabled)
 }
 
 #ifdef VBOX_STRICT
-# include <iprt/string.h>
 /**
  * Dumps a descriptor.
  *
@@ -1254,7 +1449,7 @@ VMMR0DECL(int) HWACCMR0LeaveSwitcher(PVM pVM, bool fVTxDisabled)
  * @param   Sel     Selector number.
  * @param   pszMsg  Message to prepend the log entry with.
  */
-VMMR0DECL(void) HWACCMR0DumpDescriptor(PX86DESCHC pDesc, RTSEL Sel, const char *pszMsg)
+VMMR0DECL(void) HWACCMR0DumpDescriptor(PCX86DESCHC pDesc, RTSEL Sel, const char *pszMsg)
 {
     /*
      * Make variable description string.
@@ -1263,7 +1458,7 @@ VMMR0DECL(void) HWACCMR0DumpDescriptor(PX86DESCHC pDesc, RTSEL Sel, const char *
     {
         unsigned    cch;
         const char *psz;
-    } const aTypes[32] =
+    } const s_aTypes[32] =
     {
 # define STRENTRY(str) { sizeof(str) - 1, str }
 
@@ -1326,8 +1521,8 @@ VMMR0DECL(void) HWACCMR0DumpDescriptor(PX86DESCHC pDesc, RTSEL Sel, const char *
     char        szMsg[128];
     char       *psz = &szMsg[0];
     unsigned    i = pDesc->Gen.u1DescType << 4 | pDesc->Gen.u4Type;
-    memcpy(psz, aTypes[i].psz, aTypes[i].cch);
-    psz += aTypes[i].cch;
+    memcpy(psz, s_aTypes[i].psz, s_aTypes[i].cch);
+    psz += s_aTypes[i].cch;
 
     if (pDesc->Gen.u1Present)
         ADD_STR(psz, "Present ");
@@ -1373,9 +1568,10 @@ VMMR0DECL(void) HWACCMR0DumpDescriptor(PX86DESCHC pDesc, RTSEL Sel, const char *
  * Formats a full register dump.
  *
  * @param   pVM         The VM to operate on.
+ * @param   pVCpu       The VMCPU to operate on.
  * @param   pCtx        The context to format.
  */
-VMMR0DECL(void) HWACCMDumpRegs(PVM pVM, PCPUMCTX pCtx)
+VMMR0DECL(void) HWACCMDumpRegs(PVM pVM, PVMCPU pVCpu, PCPUMCTX pCtx)
 {
     /*
      * Format the flags.
@@ -1420,7 +1616,7 @@ VMMR0DECL(void) HWACCMDumpRegs(PVM pVM, PCPUMCTX pCtx)
     /*
      * Format the registers.
      */
-    if (CPUMIsGuestIn64BitCode(pVM, CPUMCTX2CORE(pCtx)))
+    if (CPUMIsGuestIn64BitCode(pVCpu, CPUMCTX2CORE(pCtx)))
     {
         Log(("rax=%016RX64 rbx=%016RX64 rcx=%016RX64 rdx=%016RX64\n"
             "rsi=%016RX64 rdi=%016RX64 r8 =%016RX64 r9 =%016RX64\n"
