@@ -1,4 +1,4 @@
-/* $Id: tstRunTestcases.cpp 14831 2008-11-30 10:31:16Z vboxsync $ */
+/* $Id: tstRunTestcases.cpp $ */
 /** @file
  * tstRunTescases - Driver program for running VBox testcase (tst* testcase/tst*).
  */
@@ -62,6 +62,7 @@ static const char  *g_apszExclude[] =
     "testcase/tstDir",              /* useless, requires parameters */
     "testcase/tstDir-2",            /* useless, requires parameters */
     "testcase/tstGlobalConfig",
+    "testcase/tstHostHardwareLinux", /* must be killed with CTRL-C */
     "testcase/tstLdr-2",
     "testcase/tstLdr-3",
     "testcase/tstLdr",
@@ -69,11 +70,14 @@ static const char  *g_apszExclude[] =
     "testcase/tstLdrObj",
     "testcase/tstLdrObjR0",
     "testcase/tstMove",
+    "testcase/tstR0ThreadPreemption.r0", /* r0 driver, not directly executable */
     "testcase/tstRunTestcases",
+    "testcase/tstRTS3",             /* requires parameters <access key>, <secret key> */
     "testcase/tstSDL",
     "testcase/tstTime-3",
     "testcase/tstSeamlessX11",
     "testcase/tstVBoxControl",
+    "testcase/tstVDCopy",           /* requires parameters <hdd1>, <hdd2> */
     "./tstRunTestcases",
     "./tstAnimate",
     "./tstAPI",
@@ -84,11 +88,13 @@ static const char  *g_apszExclude[] =
     "./tstVBoxDbg",
     "./tstVMM-2",
     "./tstTestServMgr",
+    "./tstPDMAsyncCompletion",      /* requires parameters <source>, <dest> */
     "./tstXptDump",
     "./tstnsIFileEnumerator",
     "./tstSimpleTypeLib",
     "./tstTestAtoms",
     "./tstXptLink",
+    "./tstXPCOMCGlue",              /* user interaction required */
     "./tstTestCallTemplates",
 #if 1 // later
     "testcase/tstIntNetR0",
@@ -198,7 +204,7 @@ static void Process(const char *pszFilter, const char *pszDir)
                         if (rc != VERR_PROCESS_RUNNING)
                             break;
                         RTTIMESPEC Now;
-                        if (RTTimeSpecGetMilli(RTTimeSpecSub(RTTimeNow(&Now), &Start)) > 60*1000 /* 1 min */)
+                        if (RTTimeSpecGetMilli(RTTimeSpecSub(RTTimeNow(&Now), &Start)) > 120*1000 /* 1 min */)
                         {
                             RTPrintf("*** %s: FAILED - timed out. killing it.\n", pszTestcase);
                             RTProcTerminate(Process);
@@ -263,10 +269,10 @@ int main(int argc, char **argv)
     if (argc == 1)
     {
         char szPath[RTPATH_MAX];
-        int rc = RTPathProgram(szPath, sizeof(szPath) - sizeof("/.."));
+        int rc = RTPathExecDir(szPath, sizeof(szPath) - sizeof("/.."));
         if (RT_FAILURE(rc))
         {
-            RTPrintf("fatal error: RTPathProgram -> %Rrc\n", rc);
+            RTPrintf("fatal error: RTPathExecDir -> %Rrc\n", rc);
             return 1;
         }
         rc = RTPathSetCurrent(strcat(szPath, "/.."));
